/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;

using namespace Moses1;

int main(int argc, char* argv[]) {

		cout << "Holy Moses..." << endl;

		TStyle* plotstyle = (TStyle*)gROOT->GetStyle("Plain");
		plotstyle->cd();

		// declare the application
		TApplication theApp("App", &argc, argv);

		commonOperations commonOps; // set the class for commonOperations call
		// set styles
		commonOps.SetOKStyle();


		cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
		parameters prm; // create an instance of the parameters class
		physicalParameters physPrm; // create an instance of the physical parameters struct
		runParameters runPrm; // create an instance of the run parameters struct
		prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
		prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
		const int runtime=30;
		double totalSigmaArray [runtime];
		double g_mGluonStarArray [runtime];
			string dirName = "/KKGvsSMGstudy/";

	string treeFilePath = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_theoryTree.root");

  for(int g_mGluonStar_i=1; g_mGluonStar_i < runtime ; g_mGluonStar_i++){
		totalSigmaArray[g_mGluonStar_i]=0;
		g_mGluonStarArray[g_mGluonStar_i]=((double)g_mGluonStar_i)*0.1;
		cout << "Holy Moses..." << g_mGluonStar_i << endl;

	/*double mStar = physPrm.mStar;
	cout << "choose: m^*=";
	cin >> mStar;
	physPrm.mStar = mStar;*/

	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for :
	int idBosons[] = {PDTGLU,PDTZ,PDTWPLUS}; // glu,  Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here


	// create relevant classes instances
	couplings cplng; // set the class for couplings call
	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	lhapdf lha; // set the class for lhapdf call
	couplings masiveGcouplings;

	newZresonance zPrime("chi"); ////////////////?????????????????????
	//--------------------------------------------------------------------//

///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////
	// for the runs
	dcomplex fullAmp_SM (0,0);
	dcomplex fullAmp_GKK (0,0);
	dcomplex fullAmp_GKKSM (0,0);

	dcomplex fullAmp_Zprime (0,0);

    dcomplex unitC (1,0);
	int nMax_ysegments = 100; // for the integration on the rapidity
	int nKKmodes = 1; // only first kk mode.
	double differentialSquareAmp_SM = 0;
	double differentialSquareAmp_KK = 0;
	//double differentialSquareAmp_GKKSM = 0;


	double squareAmp_SM = 0; //masss less gloun
	double squareAmp_KK = 0;
//	double squareAmp_GKKSM = 0;

	double totalwidth_GKK[3][9]; //2 helcity states 8 lambda metrixces,6 quarks, ([2][0][6] = 0)
	double totalwidth_Z0 = 0;

	double qqbarRapidity_shat = 0; // this is the rapidity integral factor (single integral)
	customParticle outFermion1 = CUSTOMfermions[5]; // propagator will decay into a PDTTOP PDTTOP_bar pair
	customParticle outFermion2 = CUSTOMfermions[5];
	// the integral over all decay angles
	// [1] theta: distributing like ~ (1+/-cos(theta))^2
	// [2] phi: distributing uniformly
	double angularIntegral = 2*physPrm.pi * 8/3;
	physPrm.g_mGluonStar = ((double)g_mGluonStar_i)*0.1;

	double mHat_min = floor(2.05*CUSTOMfermions[5].pdt_prt.mass);
	double mHat_max = 6000.;
	double dmHat = 1.;
	double mHat = mHat_min;
	double s_hat = mHat_min*mHat_min;
	Int_t nentries = (Int_t)((mHat_max-mHat_min)/dmHat);
	cout << "nentries=" << nentries << endl;

	// initialize lha (lhapdf instance)
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name

	// alocate arrays for the graph:
	double *Sqrt_s_hat = (double *)malloc(nentries*sizeof(double));
	Double_t *Sqare_amp_SM = (Double_t *)malloc(nentries*sizeof(Double_t));
	double *Sqare_amp_KK = (double *)malloc(nentries*sizeof(double));


	TFile *treeFile = new TFile(treeFilePath.c_str(),"RECREATE");
	string treeName = "theoryTree";
	TTree *theoryTree = new TTree(treeName.c_str(),treeName.c_str());

	theoryTree->Branch("mHat",&mHat,"mHat/D");
	theoryTree->Branch("dsigmadmHat_SM",&squareAmp_SM,"squareAmp_SM/D");
	theoryTree->Branch("dsigmadmHat_KK",&squareAmp_KK,"squareAmp_KK/D");
	//theoryTree->Branch("dsigmadmHat_KK",&squareAmp_GKKSM,"squareAmp_GKKSM/D");

	cout << "\n\nRun over s_hat ([" << mHat_min << "->" << mHat_max << "]GeV)^2:" << endl;

				//for (double hOut1=-0.5 ; hOut1<=0.5 ; hOut1++) { // loop over incoming helicity states (all quarks)
				//	for (double =-0.5 ; hOut2<=0.5 ; hOut2++) { // loop over outgoing helicity states (all quarks)


	double hOut1=-0.5 ;
	double hOut2=+0.5 ;
	int Group = 0;


	double tempS=-1.;     //  tempS=-1.  this ignors the kinematic condition.
	/////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////
	// total width calculation s-channel in_f = out_f
	////////////////////////////////////////////////////////////////////////////////
			for (int KKmode=1 ; KKmode<=nKKmodes ; KKmode++) { // loop over all KK modes .one KK mode only for this model
				for ( Group=0 ; Group<3 ; Group++){
					for (int Lambda_I=1 ; Lambda_I<9 ; Lambda_I++){ //running over gell mann metrixecs // every lambda needs a tota width
					   ////////////////////////////////////////////////////////////
					   //////////////////////////starting to sum per Massive - G/////////////////////////////
					   totalwidth_GKK[Group][Lambda_I] = 0;
					   //	cout << "Group ([" << Group << "Lambda_I" << Lambda_I << ":" << endl;
						switch  (Group){
						 		case 0:{
									hOut1=-0.5;
									hOut2=-0.5;

									for (int i=0 ; i<6; i++) { // running over all of out going q - qbar pairs = s-channel.

									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
										totalwidth_GKK[Group][Lambda_I] += wdt.getpartialMassiveGWidth(Lambda_I,KKmode,tempS,
																		 PDTbosons[0], CUSTOMfermions[i], hOut1,hOut2,
																		 physPrm, masiveGcouplings);
									}//for

								}//case
								case 1:{
									hOut1=+0.5;
									hOut2=+0.5;
									for (int i=0 ; i<6; i++) { // running over all of out going q - qbar pairs = s-channel.
										i++; // down type
									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
										totalwidth_GKK[Group][Lambda_I] += wdt.getpartialMassiveGWidth(Lambda_I,KKmode,tempS,
																		 PDTbosons[0], CUSTOMfermions[i], hOut1,hOut2,
																		 physPrm, masiveGcouplings);
									}//for

								}//case
								case 2:{
									hOut1=+0.5;
									hOut2=+0.5;
									//	double tempS=0;     //  tempS=0  this ignors the kinematic condition.
									for (int i=1 ; i<6; i++) { // running over all of out going q - qbar pairs = s-channel.
										i++; // up type
										totalwidth_GKK[Group][Lambda_I] += wdt.getpartialMassiveGWidth(Lambda_I,KKmode,tempS,
																		 PDTbosons[0], CUSTOMfermions[i], hOut1,hOut2,
																		 physPrm, masiveGcouplings);
									}//for

								}//case
							}//switch
					       	cout << "Group - " << Group << " Lambda_I" << Lambda_I << ":" << "total width is " << totalwidth_GKK[Group][Lambda_I] << " GeV" << endl;

					}//for Lambda_I
			  }//for (double Group
		}//for KKmode



	int n;
	for (n=0 ; n<nentries ; n++) { // loop over s_hat

		squareAmp_SM = 0; // before each new s_hat it needs to be initialized to '0'
		squareAmp_KK = 0;
	//	squareAmp_GKKSM = 0;
		totalwidth_Z0 = 0; // before each new s_hat it needs to be initialized to '0'

		// calculate the total width of Z/photon* for each s_hat:
		// calculate the partial width to this pair of ffbar and sum it to get the total width

		///////////////////////////////////////////////////////////////////////////
		//////////////////////////z0 width///////////////////////////////
		for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
					totalwidth_Z0     += wdt.getpartialWidth(0, s_hat, PDTbosons[1], CUSTOMfermions[i], physPrm); // Z0 total width

					//totalwidth_Zprime += zPrime.getZprimeWidth(doSMcouplings, CUSTOMfermions[i]);

			}
		////////////////////////////////////////////////////////////////////////////

		for (int q=0 ; q<6 ; q++) { // run over all incoming quarks pairs include  ttbar
			differentialSquareAmp_SM = 0; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_KK = 0; // new kk only
		//	differentialSquareAmp_GKKSM = 0; // kk with sm

			for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
				for (double  hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over incoming helicity states (all quarks)


					fullAmp_SM     = dcomplex(0,0); // before each helicity state this has to be initialized to '(0,0)'


					// ---------- SM terms ----------------------------------------------------------------------
					//photon propagator
					fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, 0, hIn, hOut,
										CUSTOMfermions[q], outFermion1,
										PDTbosons[1], physPrm);
					//Z0 propagator
					fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, totalwidth_Z0, hIn, hOut,
										CUSTOMfermions[q], outFermion1,
										PDTbosons[1], physPrm);
					//SM gloun
					double number_of_glouns = 2.;					 
					fullAmp_SM += number_of_glouns*hAmp.gethelicitySMglounAmplitude(s_hat, PDTbosons[0], physPrm, outFermion1);

					// zero mass gloun

					/// total GKK calculatian per prosecc
					//    gethelicityMassiveGAmplitude(unsigned int Lambda_I,unsigned int nMode, double s,
					//                                 double totalWidth_s, double inHelicity1,double inHelicity2,
					//                                 double outHelicity1, double outHelicity2, customParticle &inFermion1,
					//                                 customParticle &inFermion2, customParticle &outFermion1, customParticle &outFermion2,
					//                                 pdtParticle &propagator
					//                                , physicalParameters &physPrm,couplings &Gcouplings){
						// 0.11762 im the z mass

					/////////////////////////totalwidth_GKK[Group][Lambda_I]
				// ---------- KK terms ----------------------------------------------------------------------
					fullAmp_GKK = dcomplex(0,0); // Masssive G total width with adding the SM width  as well
				//   tt bar only... so no  need to run
				//		for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) { // running over all of out going q - qbar pairs = s-channel.

					// KK-G amp. this will add the amplitude depending on the massive gloun group.


					// if the Helicity then the massive G can be produce.
					if (hIn == hOut){

						 if (hIn < 0.) {
							Group = 0;
						 }//if
						 else if  (((q == 1)||(q==3)||(q==5))&&(hIn > 0.)) {
							Group = 1; //down type r ;
						 }//if
						 else if  (((q == 0)||(q==2)||(q==4))&&(hIn > 0.)){
							Group = 2; //up type r ;
						 }//if
						 for (int Lambda_I =1 ;Lambda_I <9;Lambda_I++){ //summing over all of the glouns in this group
							fullAmp_GKK += hAmp.gethelicityMassiveGAmplitude(Lambda_I, nKKmodes, s_hat, totalwidth_GKK[Group][Lambda_I], hIn, hIn, hOut, hOut, CUSTOMfermions[q],CUSTOMfermions[q],outFermion1, outFermion2,PDTbosons[0],physPrm,masiveGcouplings);
					 }//for
					}//if Hin== Hout
					//fullAmp_GKKSM =fullAmp_SM + fullAmp_GKK;

					differentialSquareAmp_KK     += (s_hat*s_hat) * (1./4.) * real( fullAmp_GKK*conj(fullAmp_GKK) );
					differentialSquareAmp_SM     += (s_hat*s_hat) * (1./4.) * real( fullAmp_SM*conj(fullAmp_SM) );
					//differentialSquareAmp_GKKSM   = (s_hat*s_hat) * (1./4.) * real( fullAmp_GKKSM*conj(fullAmp_GKKSM) );
					// ------------------------------------------------------------------------------------------
					// get the differential square amplitude: |fullAmp|^2



				} // end hOut
			} // end hIn

			qqbarRapidity_shat = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha); // the qqbar rapidity integral factor
			// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
			// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
			//     and here only 3 are valid sice the propogator is colorless
			// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams
			differentialSquareAmp_SM     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
			differentialSquareAmp_KK     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
			//differentialSquareAmp_GKKSM     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;

			squareAmp_SM     += differentialSquareAmp_SM;  // only then add the differential square amplitude to the full one
			squareAmp_KK     += differentialSquareAmp_KK;
			//squareAmp_GKKSM   += differentialSquareAmp_GKKSM;

		} // end q

		// multiply by the coefficients appearing before the squared helicity amplitude
		///////////////////?????????????????????????????????????/ nc nc nc nc ????????????????????????????????????????
		squareAmp_SM     *= 0.1*0.1/(4.*s_hat) * outFermion1.Nc * angularIntegral;
		//squareAmp_SM     *= 1./(4.*s_hat) * outFermion1.Nc * angularIntegral;
		squareAmp_KK     *= 1./(4.*s_hat) * angularIntegral;
		//squareAmp_GKKSM     *= 1./(4.*s_hat) * outFermion1.Nc * angularIntegral;
		// [squareAmp] = [1/GeV^4]
		squareAmp_SM     *= 2*sqrt(s_hat); // dSigma(s_hat) -> dSigma(sqrt(s_hat)) Jacobian of the transformation
		squareAmp_KK     *= 2*sqrt(s_hat);
		//squareAmp_GKKSM     *= 2*sqrt(s_hat);
		// [squareAmp] = [1/GeV^3]
		squareAmp_SM     *= physPrm.GeV2nbarn; // to convert the units to nb/GeV (originally was nb/GeV^2 but here it is nb/GeV)
		squareAmp_KK     *= physPrm.GeV2nbarn;
		//squareAmp_GKKSM     *= physPrm.GeV2nbarn;

			// [squareAmp] = [nb/GeV]
		// see more details about the conversion using the method "prm.printUnits()" in the end of the loop.

		// fill arrays of graphs:
		Sqare_amp_SM[n]     = squareAmp_SM;
		Sqare_amp_KK[n]     = squareAmp_KK;
	//	Sqare_amp_GKKSM[n]  = squareAmp_GKKSM;
		if(squareAmp_SM<=0 || squareAmp_KK<=0)
		{
			cout << "n=" << n << ", squareAmp_KK=" << squareAmp_KK << ", squareAmp_SM=" << squareAmp_SM << endl;
		}

		//if (mHat>2500. && mHat<3500.)
		//{i
		//	cout << mHat << "\t" << squareAmp_KK << endl;
		//}

		Sqrt_s_hat[n] = mHat;

		theoryTree->Fill();

		// cout << "sqrt(s_hat)=" << mHat << "\r" << flush;
		mHat += dmHat;
		s_hat = mHat*mHat;

	} // end sMat
	prm.printUnits(); // just a reminder about units...
	theoryTree->Print();
	treeFile->Write();
	double totalSigma;
    totalSigma = commonOps.simpsonIntegration(Sqare_amp_KK , Sqrt_s_hat, n ,1000.0,5999.0);
	cout << " totalSigma " << totalSigma << " physPrm.g_mGluonStar = " << physPrm.g_mGluonStar << endl;
	totalSigmaArray[g_mGluonStar_i]=totalSigma;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////


	stringstream out;
	out << g_mGluonStar_i;
	cout << "\n\n" << "--------------creating graph--------------" << endl;
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"avi"+out.str()+"_invMass_theory.root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"avi"+out.str()+"_invMass_theory.eps");
	TFile *rootFile = new TFile(rootFileName.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph

	TCanvas *c1 = new TCanvas("c1","qqbar-->(G+gloun)-->T Tbar  dSigma(s)/ds                        ",0,0,1000,600);
	c1->SetLogy(1);

	//c1->SetGrid();
	TGraph *crossSection_SM = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_SM);
	TGraph *crossSection_KK = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_KK);

	//crossSection_SM->SetMinimum( 0.9*crossSection_SM->GetMinimum() );
	//crossSection_SM->SetMaximum( 1.1*crossSection_KK->GetMaximum() );

//	crossSection_SM->SetTitle("dSigma(s)/ds[pp-->mu-mu+]   SM vs. E6 vs. KK");
	out.str("");
	out.clear();
    out <<"(FCGB factor ="<<physPrm.g_mGluonStar <<",total cross-section in nb = " << totalSigma <<")";

	
    string stitle;
	stitle = "dSigma(s)/ds[pp-->ttbar] , SM (g + Z + gamma) vs. FCGB " + out.str();
	crossSection_SM->SetTitle(stitle.c_str());
//	crossSection_SM->SetLineColor(kRed);
	crossSection_SM->SetLineStyle(5);
	crossSection_SM->GetXaxis()->SetTitle("#sqrt{#hat s} [GeV]");
	crossSection_SM->GetYaxis()->SetTitle("#frac{d#sigma}{d#sqrt{#hat s}} [#frac{nb}{GeV}]");
	//crossSection_SM->GetYaxis()->Sqare_amp_SM[n]
	crossSection_SM->Draw("AC");

//	crossSection_Zprime->SetLineStyle(3);
//	crossSection_Zprime->SetLineColor(kGreen);
//	crossSection_Zprime->Draw("C");

	crossSection_KK->SetLineStyle(1);
//	crossSection_KK->SetLineColor(kBlack);
	crossSection_KK->Draw("C");

	//crossSection_GKKSM->SetLineStyle(1);
	//crossSection_GKKSM->SetLineColor(kGreen);
	//crossSection_GKKSM->Draw("C");

	TLegend *leg = new TLegend(0.6, 0.7, 0.98, 0.9);
//	leg->SetFillColor(5);
	leg->AddEntry(crossSection_SM,"SM","L");
	    string text1;
    ostringstream ss;
    ss <<physPrm.g_mGluonStar <<",total cross-section in nb = " << totalSigma; ;
    text1 = ss.str();
    text1 = "residal LS a: "+text1;
    TText *th1 = new TText(23.33,0.6,text1.c_str());
	th1->SetTextSize(0.025);
	TBox *box = new TBox(23.,0.3,42.,0.75);
	box->SetLineColor(1);
	box->SetLineWidth(2);
	leg->AddEntry(crossSection_KK,"FCGB","L");
	//leg->AddEntry(crossSection_GKKSM,"G_{KK}+SM","L");
	leg->Draw();
	//pt1->SetLabel("Plot information");
	//pt1->Draw("same");
    //pave->Draw("same");

	//c1->SetLogy();

	c1->Print(rootFileName.c_str(),"root");
	th1->Draw();
	box->Draw();
	c1->Update();
	
	c1->Print(epsFileName.c_str());
	rootFile->Write();

	cout << "\n" << "done..." << endl;

	// wait so the plots can hold on
      //  c1->WaitPrimitive();
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	stringstream out;
	out << "SigmaVSg" ;
	cout << "\n\n" << "--------------creating graph--------------" << endl;
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"avi"+out.str()+"_invMass_theory.root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"avi"+out.str()+"_invMass_theory.eps");
	TFile *rootFile = new TFile(rootFileName.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph
	TCanvas *c1 = new TCanvas("c1","Inegrated cross Section Vs FCGB coupling",0,0,1000,600);
	//c1->SetLogy(1);
	//c1->SetGrid();
	TGraph *crossSection_total= new TGraph (runtime, g_mGluonStarArray, totalSigmaArray);
    string stitle;
	stitle = "Inegrated cross Section Vs FCGB coupling" ;
	crossSection_total->SetTitle(stitle.c_str());
//	crossSection_total->SetLineColor(kRed);
	crossSection_total->SetLineStyle(1);
	crossSection_total->GetXaxis()->SetTitle("g (FCGB)");
	crossSection_total->GetYaxis()->SetTitle("#sigma [nb]");
	//crossSection_total->GetYaxis()->Sqare_amp_SM[n]
	crossSection_total->Draw("AC");
	TLegend *leg = new TLegend(0.6, 0.7, 0.98, 0.9);
//	leg->SetFillColor(5);
	leg->AddEntry(crossSection_total,"Sigma","L");
	leg->Draw();
	c1->Print(rootFileName.c_str(),"root");
	c1->Update();	
	c1->Print(epsFileName.c_str());
	rootFile->Write();
	cout << "\n" << "done..." << endl;
	// wait so the plots can hold on
      //  c1->WaitPrimitive();
	
	return 0;
}

