//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------
using namespace std;
using namespace Moses1;


int main() {
	cout << "Holy Moses..." << endl;
	
	// set styles
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();
	//gStyle->SetOptStat(11);
	//gStyle->SetErrorX(0.5);
	
	// declare some histos and related root entities
	commonOperations commonOps;
	string outFileName = "/SOFTWAREstudy/KK_gammaZ";
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".eps");

	// open the input files
	string infile = commonOps.checkANDsetFilepath("MOSESDATA","/pythiaKaluzaKlein_muons.dat");
	ifstream muFile;
	muFile.open(infile.c_str());

	// set the root file
	TFile *histFile = new TFile(rootFileName.c_str(),"recreate");
	TCanvas *c1 = new TCanvas("c1","Z0",0,0,1400,800);
	c1->SetLogy();
	c1->SetFillColor(0);

	// binning
	Double_t max_mZ = 6000.;
	Double_t min_mZ = 1000.;
	Int_t nBins_mZ = 100;
	
	// histos
	TH1 *mZ_hist = new TH1D("Pythia-KK: m(mu-,mu+)", "Pythia-KK: muonpairs invariant mass", nBins_mZ, min_mZ, max_mZ);
	//mZ_hist->SetLineColor(kRed);
	//mZ_hist->SetFillColor(kRed);
	mZ_hist->SetLineWidth(2);
	mZ_hist->SetXTitle("m(mu-mu+)[GeV]");
	mZ_hist->SetYTitle("Events");
	mZ_hist->SetTitle("Muon pairs invariant mass distribution");

	// get the internal data
	double mZ;
	int evt = 0;
	int tmpEvt = 0;
	while(!muFile.eof()) {
		muFile >> evt;
		if(evt != tmpEvt+1) {cout << tmpEvt << " events were recived" << endl; break;}
		else {
			tmpEvt++;
			muFile >> mZ;
			mZ_hist->Fill( (Double_t)mZ );
		}
	}
	muFile.close();

	// resolve the theoretical cross section line
        // alocate arrays for the graph and open input file:
	Int_t nGraphEntries = (Int_t)max_mZ - (Int_t)(min_mZ);
        Double_t *mHat = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	Double_t *sigma = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
        Double_t *nEvents = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	string infile_sigmaTheory = commonOps.checkANDsetFilepath("MOSESDATA","/sigma.KK");
        ifstream inputFile;
        inputFile.open(infile_sigmaTheory.c_str());
	// get the data
        int n=0;
        double tmp_mHat = min_mZ;
        double tmp_sigma;
        while(!inputFile.eof() && n<=nGraphEntries) {
                inputFile >> tmp_mHat;
                inputFile >> tmp_sigma;
                if (tmp_mHat>=min_mZ && tmp_mHat<=max_mZ) { // read only the desired block
                        while (!inputFile.eof() && n<=nGraphEntries) {
                                if (n==0) {
                                        mHat[n] = (Double_t)tmp_mHat;
                                        sigma[n] = (Double_t)tmp_sigma;
                                }
				else {
                               		inputFile >> mHat[n];
                                	inputFile >> sigma[n];
				}
				if (n==nGraphEntries) {break;}
                                n++;
                        }
                }
        }
	inputFile.close();

	// calculate the integral of the theoretical line (simpson integration method)
	int nSegments = (int)nGraphEntries-1;
        Double_t dmHat = 1.;
        Double_t h = dmHat/3;
        Double_t integrand = 0;
        Double_t sigmaTot = 0;
        int i=0;
        while (i<=nSegments) {
                // calculate the integrand:
                integrand = sigma[i];
                // simpson:
                if (i==0 || i==nSegments) {sigmaTot += integrand;}
                else if (i%2==0 && i!=0 && i!=nSegments) {sigmaTot += 2*integrand;}
                else {sigmaTot += 4*integrand;}
                // propagate i:
                i++;
        }
        sigmaTot *= h; // [nb]
	Double_t nb2mb = 1e-6;
	sigmaTot *= nb2mb; // [mb]
        Int_t Ntot = (Int_t)mZ_hist->GetEntries(); // [events]
	cout << "sigmaTot=" << sigmaTot << "[mb]" << " for " << Ntot << " events" << endl;
        Double_t luminosity = Ntot/sigmaTot; // [events/mb]
        Double_t binSize = (max_mZ - min_mZ)/nBins_mZ; // [GeV]

	// fill the nEvents array (theoretical line) with the transformed cross section values: sigma[mb]->sigma[events]
	for (int j=0 ; j<(int)nGraphEntries ; j++) {
		nEvents[j] = (sigma[j] * nb2mb) * (binSize * luminosity); // ([mb/GeV]) * ([GeV] * [events/mb]) = [events]
	}


	// declare the graph for the theoreticl line
        TGraph *crossSection = new TGraph (nGraphEntries, mHat, nEvents);
        crossSection->SetLineColor(kGreen);
	crossSection->SetLineWidth(2);

        // legends
	TLegend *leg1 = new TLegend(0.65,0.78,0.98,0.88);
	leg1->AddEntry(mZ_hist,"Pythia8 external Kaluza-Klein production", "L");
	leg1->AddEntry(crossSection,"Theoretical Kaluza-Klein Cross-section","L");
	leg1->SetFillColor(5);

	// draw
	gStyle->SetOptStat(11);
	mZ_hist->Draw();
	crossSection->Draw("C");
	leg1->Draw("sames");
	c1->Update();

	c1->Print(rootFileName.c_str(),"root");
	c1->Update();
	c1->Print(epsFileName.c_str());
	histFile->Write();

	// Done.
	return 0;
}
