// pythia_internalVSexternal.cxx is a part of Moses,
// a framework of interfaces for developing HEP models.
// Copyright (C) 2010 Noam Hod and Mark Sutton
//
// Moses is licenced under version 2 of the GPL,
// see COPYING for details.
// Please respect the MCnet academic usage guidelines,
// see http://www.montecarlonet.org/GUIDELINES for details.


// This is an example program to analyze the differences 
// between the the internal and external implementations of
// the qqbar -> gamma/Z^0 -> mu+mu- hard process (only).


//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace Moses1;


int main(int argc, char ** argv) {
	cout << "Holy Moses..." << endl;

	string flag = "";
	string abstract = "";
	if(argc>1) {
		string flag(argv[1]);
		string abstract(argv[2]);
		if (flag=="-s" && abstract=="yes")
		{
			cout << "\t +----------------------- process summary ---------------------------------------------------------+" << endl;
			cout << "\t | (1) description of program:                                                                     |" << endl;
			cout << "\t |     This is an example program to analyze the differences                                       |" << endl;
			cout << "\t |     between the the internal and external implementations of                                    |" << endl;
			cout << "\t |     the qqbar -> gamma/Z^0 -> mu+mu- hard process (only).                                       |" << endl;
			cout << "\t | (2) description of the user inputs:                                                             |" << endl;
			cout << "\t |     - $MOSESDATA/pythia_internal_muons.root (output of SM_qqbar2gmZ2mumubar_pythiaInternalMain) |" << endl;
			cout << "\t |     - $MOSESDATA/pythia_external_muons.root (output of SM_qqbar2gmZ2mumubar_pythiaMain)         |" << endl;
			cout << "\t |     - $MOSESDATA/dsigmadmHat_SM_theory.root (output of SM_qqbar2gmZ2mumubar_theoryMain)         |" << endl; 
			cout << "\t | (3) description of outputs:                                                                     |" << endl;
			cout << "\t |     - $MOSESDATA/SM_gammaZ0_internalVSexternal.root                                             |" << endl;
			cout << "\t |     - $MOSESDATA/SM_gammaZ0_internalVSexternal.eps                                              |" << endl;
			cout << "\t +----------------------- process summary ---------------------------------------------------------+" << endl;
			return 0;
		}
	}
	
	TStyle* s = (TStyle*)gROOT->GetStyle("Plain");
	s->cd();

	commonOperations commonOps;
	// set the style
	commonOps.SetOKStyle();

	
	
	// declare some histos and related root entities
	string dirName = "/";

	string outFileName = dirName+"SM_gammaZ0_internalVSexternal";
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA", outFileName+".eps");


	string rootFileName1 = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"pythia_internal_muons.root");
	commonOps.exitIfNotExist(rootFileName1);
	string treeName1 = "testTree";
	TFile *treeFile1 = new TFile(rootFileName1.c_str());
	TTree *tree1 = (TTree*)treeFile1->Get(treeName1.c_str());
	int eventNumber;
	double mass, theta;
	tree1->SetBranchAddress("eventNumber",&eventNumber);
	tree1->SetBranchAddress("mHat",&mass);
	tree1->SetBranchAddress("thetaHat",&theta);


	string rootFileName2 = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"pythia_external_muons.root");
	commonOps.exitIfNotExist(rootFileName2);
	string treeName2 = "testTree";
	TFile *treeFile2 = new TFile(rootFileName2.c_str());
	TTree *tree2 = (TTree*)treeFile2->Get(treeName2.c_str());
	tree2->SetBranchAddress("eventNumber",&eventNumber);
	tree2->SetBranchAddress("mHat",&mass);
	tree2->SetBranchAddress("thetaHat",&theta);


	// set the root file
	TFile *histFile = new TFile(rootFileName.c_str(),"recreate");
	TCanvas *c1 = new TCanvas("c1","Z0",0,0,1400,800);
	c1->Divide(2,2);

	// plots of analysis on muon pairs
	TVirtualPad *pad1=c1->cd(1);
	pad1->SetLogy();

	TVirtualPad *pad2=c1->cd(1);
	pad2->SetLogy();


	// binning
	Int_t nBins_mZ0 = 180;
	Double_t min_mZ0 = 20.;
	Double_t max_mZ0 = 200.;

	Int_t nBins_costheta = 120;
	Double_t min_costheta = -1.2;
	Double_t max_costheta = 1.2;
	

	// invariant mass histos
	TH1 *mZ0_intrn = new TH1D("", "", nBins_mZ0, min_mZ0, max_mZ0);
	mZ0_intrn->SetFillStyle(3002);
	mZ0_intrn->SetXTitle("#sqrt{#hat{s}}(#mu^{-}#mu^{+}) [GeV]");
	mZ0_intrn->SetYTitle("Events");

	TH1 *mZ0_extrn = new TH1D("", "", nBins_mZ0, min_mZ0, max_mZ0);
	mZ0_extrn->SetLineStyle(3);

	
	// costheta histos
	TH1 *costheta_intrn = new TH1D("", "", nBins_costheta, min_costheta, max_costheta);
	costheta_intrn->SetFillStyle(3002);
	costheta_intrn->SetXTitle("cos#theta*_{#beta}");
	costheta_intrn->SetYTitle("#frac{Events}{#sumEvents}");
	
	TH1 *costheta_extrn = new TH1D("", "", nBins_costheta, min_costheta, max_costheta);
	costheta_extrn->SetLineStyle(3);


	// invariant mass relative difference
	TH1 *positive_diff_mZ0 = new TH1D("", "", nBins_mZ0, min_mZ0, max_mZ0);
	positive_diff_mZ0->SetFillStyle(3004);
	positive_diff_mZ0->SetXTitle("#sqrt{#hat{s}}(#mu^{-}#mu^{+}) [GeV]");
	positive_diff_mZ0->SetYTitle("#frac{N_{int}-N_{ext}}{#sqrt{#delta^{2}N_{int} + #delta^{2}N_{ext}}}");

	TH1 *negative_diff_mZ0 = new TH1D("", "", nBins_mZ0, min_mZ0, max_mZ0);
	negative_diff_mZ0->SetFillStyle(3005);
	
	// costheta relative difference
	TH1 *positive_diff_costheta = new TH1D("", "", nBins_costheta, min_costheta, max_costheta);
	positive_diff_costheta->SetFillStyle(3004);
	positive_diff_costheta->SetXTitle("cos#theta*_{#beta}");
	positive_diff_costheta->SetYTitle("#frac{N_{int}-N_{ext}}{#sqrt{#delta^{2}N_{int} + #delta^{2}N_{ext}}}");
	
	TH1 *negative_diff_costheta = new TH1D("", "", nBins_costheta, min_costheta, max_costheta);
	negative_diff_costheta->SetFillStyle(3005);



	// get the internal data
	double mZ0, costheta;
	double mZ0_known = 91.18760;
	double halfRange = 2.50232/2.;
	Int_t treeNentries1 = (Int_t)tree1->GetEntries();
	for (Int_t i=0 ; i<treeNentries1 ; i++) {
		tree1->GetEntry(i);
		mZ0 = mass;
		costheta = cos(theta);
		mZ0_intrn->Fill( (Double_t)mZ0 );
		if (mZ0 >= (mZ0_known-halfRange) && mZ0 <= (mZ0_known+halfRange)) {
			costheta_intrn->Fill( (Double_t)costheta );
		}
	}

	// get the external data
	Int_t treeNentries2 = (Int_t)tree2->GetEntries();
	for (Int_t i=0 ; i<treeNentries2 ; i++) {
		tree2->GetEntry(i);
		mZ0 = mass;
		costheta = cos(theta);
		mZ0_extrn->Fill( (Double_t)mZ0 );
		if (mZ0 >= (mZ0_known-halfRange) && mZ0 <= (mZ0_known+halfRange)) {
			costheta_extrn->Fill( (Double_t)costheta );
		}
	}

	// chi2 and kolmogorov tests
	Double_t chi2       = mZ0_intrn->Chi2Test(mZ0_extrn, "UUCHI2/NDF");
	Double_t kolmogorov = mZ0_intrn->KolmogorovTest(mZ0_extrn, "N");
	cout << "mZ0 histos:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;

	chi2       = costheta_intrn->Chi2Test(costheta_extrn, "UUCHI2/NDF");
	kolmogorov = costheta_intrn->KolmogorovTest(costheta_extrn, "N");
	cout << "cosTheta histos:  chi2 = " << chi2 << "\t" << "Kolmogorov = " << kolmogorov << endl;



	
	// calculate AFB internal (no need to normalize):
	cout << "\n" << endl;
	Double_t Nplus = 0;
	Double_t Nminus = 0;
	Double_t Afb, dAfb;

	Double_t cost = min_costheta;
	Double_t dcost = (max_costheta - min_costheta)/(Double_t)nBins_costheta;
	for (Int_t i=0 ; i<nBins_costheta ; i++) {
		if (cost < 0) { Nminus += (Double_t)costheta_intrn->GetBinContent(i); }
		else if (cost >= 0) { Nplus += (Double_t)costheta_intrn->GetBinContent(i); }
		cost += dcost;
	}
	Afb = (Nplus-Nminus)/(Nplus+Nminus);
	dAfb = sqrt((1-Afb*Afb)/(Nplus+Nminus));
	cout << "Afb(int) = " << Afb << " +/-" << dAfb << "  in the range [" << mZ0_known-halfRange << "GeV," << mZ0_known+halfRange << "GeV] for " << Nplus+Nminus << " events in that range" << endl;

        // calculate AFB external (no need to normalize):
	Nplus = 0;
	Nminus = 0;

	cost = min_costheta;
	dcost = (max_costheta - min_costheta)/(Double_t)nBins_costheta;
	for (Int_t i=0 ; i<nBins_costheta ; i++) {
		if (cost < 0) { Nminus += (Double_t)costheta_extrn->GetBinContent(i); }
		else if (cost >= 0) { Nplus += (Double_t)costheta_extrn->GetBinContent(i); }
		cost += dcost;
	}
	Afb = (Nplus-Nminus)/(Nplus+Nminus);
	dAfb = sqrt((1-Afb*Afb)/(Nplus+Nminus));
	cout << "Afb(ext) = " << Afb << " +/-" << dAfb << "  in the range [" << mZ0_known-halfRange << "GeV," << mZ0_known+halfRange << "GeV] for " << Nplus+Nminus << " events in that range" << endl;
	cout << "\n" << endl;


	// resolve the theoretical cross section line
        // alocate arrays for the graph and open input file:
	string infile_sigmaTheory = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"dsigmadmHat_SM_theory.root");
	commonOps.exitIfNotExist(infile_sigmaTheory);
	string treeName3 = "testTree";
	TFile *treeFile3 = new TFile(infile_sigmaTheory.c_str());
	TTree *tree3 = (TTree*)treeFile3->Get(treeName3.c_str());
	
	// get the data from the tree
	double mHat_SM;
	double dsigmadmHat_SM;
	tree3->SetBranchAddress("mHat_SM",&mHat_SM);
	tree3->SetBranchAddress("dsigmadmHat_SM",&dsigmadmHat_SM);

	Int_t treeNentries = (Int_t)tree3->GetEntries();

	// calculate expected number of events in the peak area
	Int_t nGraphEntries = (Int_t)( max_mZ0 - min_mZ0 );
	Double_t *mHat = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	Double_t *sigma = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	Double_t *nEvents = (Double_t *)malloc(nGraphEntries*sizeof(Double_t));
	
	Int_t n=0;
	for (Int_t i=0; i<treeNentries ; i++) {

		tree3->GetEntry(i);

		if (mHat_SM>=min_mZ0 && mHat_SM<=max_mZ0) {
			while (n<nGraphEntries && (mHat_SM>=min_mZ0 && mHat_SM<=max_mZ0) && ((i+n)<treeNentries)) {
				tree3->GetEntry(i+n);
				mHat[n] = mHat_SM;
				sigma[n] = dsigmadmHat_SM;
				n++;
			}
		}
	}




	// calculate the integral of the theoretical line (simpson integration method)
	int nSegments = (int)nGraphEntries-1;
        Double_t dmHat = 1.;
        Double_t h = dmHat/3;
        Double_t integrand = 0;
        Double_t sigmaTot = 0;
        int i=0;
        while (i<=nSegments) {
                // calculate the integrand:
                integrand = sigma[i];
                // simpson:
                if (i==0 || i==nSegments) {sigmaTot += integrand;}
                else if (i%2==0 && i!=0 && i!=nSegments) {sigmaTot += 2*integrand;}
                else {sigmaTot += 4*integrand;}
                // propagate i:
                i++;
        }
        sigmaTot *= h; // [nb]
	Double_t nb2mb = 1e-6;
	sigmaTot *= nb2mb; // [mb]
        Int_t Ntot = (Int_t)mZ0_intrn->GetEntries(); // [events]
	cout << "sigmaTot=" << sigmaTot << "[mb]" << " for " << Ntot << " events" << endl;
        Double_t luminosity = Ntot/sigmaTot; // [events/mb]
        Double_t binSize = (max_mZ0 - min_mZ0)/nBins_mZ0; // [GeV]

	// fill the nEvents array (theoretical line) with the transformed cross section values: sigma[mb]->sigma[events]
	for (int j=0 ; j<(int)nGraphEntries ; j++) {
		nEvents[j] = (sigma[j] * nb2mb) * (binSize * luminosity); // ([mb/GeV]) * ([GeV] * [events/mb]) = [events]
	}

	// declare the graph for the theoreticl line
        TGraph *crossSection = new TGraph (nGraphEntries, mHat, nEvents);
        //crossSection->SetLineColor(kGreen);
	//crossSection->SetLineStyle(2);
	crossSection->SetLineStyle(8);



	// resolve the relative differences (in each bin) between the internal and the external invariant mass data
	Double_t intrn, extrn, reldiff;
	for (Int_t n=0 ; n<nBins_mZ0 ; n++) {
		intrn = mZ0_intrn->GetBinContent(n);
		extrn = mZ0_extrn->GetBinContent(n);
		if ((intrn+extrn)!=0) { // for not dividing by zero...
			reldiff = (intrn-extrn) / sqrt(intrn + extrn);
			if (reldiff < 0) { negative_diff_mZ0->SetBinContent(n,reldiff); }
			else if (reldiff > 0) { positive_diff_mZ0->SetBinContent(n,reldiff); }
			else if (reldiff == 0) {
				positive_diff_mZ0->SetBinContent(n,0);
				negative_diff_mZ0->SetBinContent(n,0);
			}
		}
		else {
			positive_diff_mZ0->SetBinContent(n,0);
			negative_diff_mZ0->SetBinContent(n,0);
		}
	}


	// calculate chi square:
	double chiSquare_tmp = 0;
	double chiSquare_cosTheta = 0.;
	double chiSquare_mZ0 = 0.;
	int fullBins = 0;

	for (Int_t i=0 ; i<nBins_mZ0 ; i++) {
		if (mZ0_extrn->GetBinContent(i) > 0) {
			fullBins++;
			chiSquare_tmp = (
						( mZ0_extrn->GetBinContent(i) - mZ0_intrn->GetBinContent(i) ) /
						sqrt( mZ0_extrn->GetBinContent(i) + mZ0_intrn->GetBinContent(i) )
					);
			chiSquare_mZ0 += chiSquare_tmp*chiSquare_tmp;
		}
		else {chiSquare_mZ0 += 0.;}
	}
	chiSquare_mZ0 /= (fullBins-1);
	chiSquare_mZ0 = sqrt(chiSquare_mZ0);
	cout << "chi square per DOF for mZ0 distribution is: " << chiSquare_mZ0 << endl;


	// resolve the relative differences (in each bin) between the internal and the external cos(theta*) data
	// first have to normalize the histos:
	Double_t normalization_intrn = 1., dNormalization_intrn = 0.;
	normalization_intrn = costheta_intrn->Integral();
        if (normalization_intrn != 0) {
		for (Int_t i=0 ; i<nBins_costheta ; i++) { dNormalization_intrn += sqrt(costheta_intrn->GetBinContent(i)); }
		costheta_intrn->Scale(1./normalization_intrn);
	}
        else {cerr << "normalization for costheta_intrn == 0" << endl;}
	Double_t normalization_extrn = 1., dNormalization_extrn = 0.;
        normalization_extrn = costheta_extrn->Integral();
        if (normalization_extrn != 0) {
		for (Int_t i=0 ; i<nBins_costheta ; i++) { dNormalization_extrn += sqrt(costheta_extrn->GetBinContent(i)); }
		costheta_extrn->Scale(1./normalization_extrn);
	}
        else {cerr << "normalization for costheta_extrn == 0" << endl;}

	fullBins = 0;
	Double_t dIntrn=0, dExtrn=0;
	for (Int_t n=0 ; n<nBins_costheta ; n++) {
		intrn = costheta_intrn->GetBinContent(n);
		// the error is **not** sqrt(intrn) since it was normalized:
		dIntrn = (1./normalization_intrn) * sqrt( intrn*normalization_intrn + (intrn*dNormalization_intrn)*(intrn*dNormalization_intrn) );
		extrn = costheta_extrn->GetBinContent(n);
		// the error is **not** sqrt(extrn) since it was normalized:
		dExtrn = (1./normalization_extrn) * sqrt( extrn*normalization_extrn + (extrn*dNormalization_extrn)*(extrn*dNormalization_extrn) );
		if ((dIntrn*dIntrn + dExtrn*dExtrn)!=0) { // for not dividing by zero...
			reldiff = (intrn-extrn) / sqrt( dIntrn*dIntrn + dExtrn*dExtrn );
			if (reldiff < 0) { negative_diff_costheta->SetBinContent(n,reldiff); }
			else if (reldiff > 0) { positive_diff_costheta->SetBinContent(n,reldiff); }
			else if (reldiff == 0) {
				positive_diff_costheta->SetBinContent(n,0);
				negative_diff_costheta->SetBinContent(n,0);
			}
			fullBins++;
			chiSquare_cosTheta += reldiff*reldiff;
		}
		else {
			positive_diff_costheta->SetBinContent(n,0);
			negative_diff_costheta->SetBinContent(n,0);
		}
	}
	chiSquare_cosTheta /= (fullBins-1);
	chiSquare_cosTheta = sqrt(chiSquare_cosTheta);
	cout << "chi square per DOF for cos(theta) distribution is: " << chiSquare_cosTheta << endl;
	


        // legends
	TLegend *leg1 = new TLegend(0.64,0.6,0.88,0.9);
	leg1->AddEntry(mZ0_intrn,"internal","L");
	leg1->AddEntry(mZ0_extrn,"external","L");
//	leg1->AddEntry(crossSection,"d#sigma/d#hat s","L");
//	leg1->SetFillColor(kBlue-10);
	
	TLegend *leg2 = new TLegend(0.4,0.6,0.64,0.9);
	leg2->AddEntry(costheta_intrn,"internal","L");
	leg2->AddEntry(costheta_extrn,"external","L");
//	leg2->SetFillColor(kBlue-10);
//	leg2->SetFillColor(kBlue-10);

	// draw
	c1->cd(1);
//	gStyle->SetOptStat(11);
	mZ0_intrn->Draw();
	mZ0_extrn->Draw("sames");
//	crossSection->Draw("C");
	leg1->Draw("sames");

/*
	c1->Update();
	TPaveStats *s1 = (TPaveStats*)mZ0_intrn->FindObject("stats");
	s1->SetY1NDC(0.818); //new y start position
	s1->SetY2NDC(0.9); //new y end position
*/

	c1->cd(2);
//	gStyle->SetOptStat(11);
	costheta_intrn->SetMaximum(1.1*costheta_intrn->GetMaximum());
	costheta_intrn->Draw();
	costheta_extrn->Draw("sames");
	leg2->Draw("sames");

/*	
	c1->Update();
	TPaveStats *s2 = (TPaveStats*)costheta_intrn->FindObject("stats");
	s2->SetY1NDC(0.818); //new y start position
	s2->SetY2NDC(0.9); //new y end position
*/

	// relative difference histos
	Double_t max1, max2, max;


	c1->cd(3);
	max1 = abs(positive_diff_mZ0->GetMinimum());
	max2 = abs(positive_diff_mZ0->GetMaximum());
	max = (max1 > max2) ? max1 : max2; // the ternary operator ( ? : )
	positive_diff_mZ0->SetMinimum(-2.*max);
	positive_diff_mZ0->SetMaximum(+2.*max);

	positive_diff_mZ0->Draw();
	negative_diff_mZ0->Draw("sames");

/*
	c1->Update();
	TPaveStats *s3 = (TPaveStats*)positive_diff_mZ0->FindObject("stats");
	s3->SetY1NDC(10.); //new y start position
	s3->SetY2NDC(10.); //new y end position

	c1->Update();
	TPaveStats *s4 = (TPaveStats*)negative_diff_mZ0->FindObject("stats");
	s4->SetY1NDC(10.); //new y start position
	s4->SetY2NDC(10.); //new y end position
*/
	c1->cd(4);
	max1 = abs(positive_diff_costheta->GetMinimum());
	max2 = abs(positive_diff_costheta->GetMaximum());
	max = (max1 > max2) ? max1 : max2; // the ternary operator ( ? : )
	positive_diff_costheta->SetMinimum(-2.*max);
	positive_diff_costheta->SetMaximum(+2.*max);

	positive_diff_costheta->Draw();
	negative_diff_costheta->Draw("sames");

/*
	c1->Update();
	TPaveStats *s5 = (TPaveStats*)positive_diff_costheta->FindObject("stats");
	s5->SetY1NDC(10.); //new y start position
	s5->SetY2NDC(10.); //new y end position
	c1->Update();
	TPaveStats *s6 = (TPaveStats*)negative_diff_costheta->FindObject("stats");
	s6->SetY1NDC(10.); //new y start position
	s6->SetY2NDC(10.); //new y end position
*/

	c1->Update();
	c1->Print(rootFileName.c_str(),"root");
	c1->Print(epsFileName.c_str());
	histFile->Write();

	// Done.
	return 0;
}
