/************************************************************/
/* class to evaluate the user defined couplings             */
/* the function here is taken from the "Z LINE SHAPE" paper */
/************************************************************/
#include "couplings.h"
using namespace std;

namespace Moses1 {

couplings::couplings(){}
couplings::~couplings(){}

// to return the couplings of a fermion with helicity lambda=+1/2 to Z0/gamma gauge bosons
double couplings::getCouplings(double helicity,
				pdtParticle &propagator, customParticle &fermion,
				physicalParameters &physPrm) {
	double g =0.;

	if (propagator.id == PDTGAMMA) {g = fermion.pdt_prt.charge;} // is photon ?
	else if (propagator.id == PDTZ) { // is Z0 boson ?
		if (helicity < 0.) { // for a fermion with negative helicity
			g = fermion.weakI3 - fermion.pdt_prt.charge*physPrm.sinThetaW2;
		}
		else { // for a fermion with positive helicity
			g = (-1.) * fermion.pdt_prt.charge * physPrm.sinThetaW2;
		}
		g *= 1./sqrt(physPrm.sinThetaW2*physPrm.cosThetaW2);
	}
	else {
		cerr << "error: in couplings.cxx: propogator with id=" << propagator.id << " is not supported" << endl;
		exit(-1);
	}
	return(g);
}


double couplings::getCouplings(unsigned int nMode, double helicity,
				pdtParticle &propagator, customParticle &fermion,
				physicalParameters &physPrm) {

	if (nMode == 0) {return getCouplings(helicity,propagator,fermion,physPrm);} // S.M couplings
	else {return sqrt(2)*getCouplings(helicity,propagator,fermion,physPrm);} // E.D: simply multiply by sqrt(2). See Tom Rizzo's paper
	//-------------------------------------------------------------------------------------------------------
}


/////////////////////////////////Massiv Gloun /////////////////////////////
dcomplex  couplings::getMassiveGlounSchannelCouplings(unsigned int Lambda_I,unsigned int nMode, double helicity1,double helicity2,
                                        pdtParticle &propagator,  customParticle &fermion1,customParticle &fermion2,
                                        physicalParameters &physPrm){
    dcomplex  g (0.,0.);
//    double gScaling = 0.99 ; // this is the relation between gGlounStar/gQCD

        gMatrix.SetGellMannMatrices();
  //  customParticle fermion2 = fermion1;  //schannel

    unsigned int fermion_1_Index = 0;
    unsigned int fermion_2_Index = 0;
    //gMatrix.gellMannMatrices();
    double uptaype1  = 0.; // false the particle is not up type
    double uptaype2  = 0.; // false the particle is not up type

    if (fermion1.pdt_prt.id == PDTUP)  {fermion_1_Index = 0; uptaype1 = 1.;} //u
    if (fermion1.pdt_prt.id == PDTDWN) {fermion_1_Index = 0; uptaype1 = -1.;} //d
    if (fermion1.pdt_prt.id == PDTCHM) {fermion_1_Index = 1; uptaype1 = 1.;} //c
    if (fermion1.pdt_prt.id == PDTSTR) {fermion_1_Index = 1; uptaype1 = -1.;} //s
    if (fermion1.pdt_prt.id == PDTBOT) {fermion_1_Index = 2; uptaype1 = 1.;} //t
    if (fermion1.pdt_prt.id == PDTTOP) {fermion_1_Index = 2; uptaype1 = -1.;} //b

    if (fermion2.pdt_prt.id == PDTUP)  {fermion_2_Index = 0; uptaype2 = 1.;}
    if (fermion2.pdt_prt.id == PDTDWN) {fermion_2_Index = 0; uptaype2 = -1.;}
    if (fermion2.pdt_prt.id == PDTCHM) {fermion_2_Index = 1; uptaype2 = 1.;}
    if (fermion2.pdt_prt.id == PDTSTR) {fermion_2_Index = 1; uptaype2 = -1.;}
    if (fermion2.pdt_prt.id == PDTBOT) {fermion_2_Index = 2; uptaype2 = 1.;}
    if (fermion2.pdt_prt.id == PDTTOP) {fermion_2_Index = 2; uptaype2 = -1.;}

   if (propagator.id ==  PDTGLU) {
                //cout << "this a gloun in getMassiveGlounSchannelCouplings " << endl; works!

                if (helicity1 == helicity2){//  for s channel, need revisite for t channel.  assume f1 = f2.
                        if ((fermion1.pdt_prt.id != PDTTOP)||(fermion1.pdt_prt.id != PDTBOT)){
                                        g = -0.2 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];  // 0.2 = 1/5 , fermion that are not top nor bottom
                        }
                        if(helicity1 < 0.){// left handed particle
                                                if (fermion1.pdt_prt.id == PDTTOP){//its top
                                                        g = 1.0 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];

                                                }//if (fermion1.pdt_prt.id == PDTTOP)
                                                else if (fermion1.pdt_prt.id == PDTBOT) {//its bottom
                                                        g = 1.0 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];
                                        //              if (g != (0.*g)){
                                        //              cout << "g  - " << g << " helicity1 - " << helicity1 << " fermion1.pdt_prt.id - A" << fermion1.pdt_prt.id << " Lambda_I "<< Lambda_I << endl;
                                        //      }

                                                }//if (fermion1.pdt_prt.id == PDTBOT)
                        }//(helicity1 < 0.)
                        else if (helicity1 > 0.){ // right handed particle
                                                if ( fermion1.pdt_prt.id == PDTTOP ){
                                                        // down type * uptaype2 = -1 if the particles are from different groups. Q_Ur and Q_Dr
                                                        g =  5.0 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];
                                        //              if (g != (0.*g)){
                                        //              cout << "g  - " << g << " helicity1 - " << helicity1 << " fermion1.pdt_prt.id - A" << fermion1.pdt_prt.id << " Lambda_I " << Lambda_I << endl;
                                        //      }
                                                }
                                                else if (fermion1.pdt_prt.id == PDTBOT){//its b
                                                        g =  0.2 * gMatrix.Lambda[Lambda_I][fermion_1_Index][fermion_2_Index];
                                        //              if (g != (0.*g)){
                                        //              cout << "g  - " << g << " helicity1 - " << helicity1 << " fermion1.pdt_prt.id - A" << fermion1.pdt_prt.id << " Lambda_I " << Lambda_I << endl;
                                        //      }
                                                }
                        }//(helicity1 > 0.)
                  }//(helicity1 == helicity2)
                    g = g * physPrm.g_mGluonStar;// * physPrm.gQCD;
		//cout << "toal g is " << g<< endl;
                  return(g);
        }else {// propagator.id ==  PDTGLU

                cout << "this is not a gloun in getMassiveGlounSchannelCouplings " << endl;
    }



        return 0.;
} //couplings::getCouplings



}

