/*****************************/
/* class to emulate the cuts */
/*****************************/
#include "cutsEmulator.h"

using namespace std;

namespace Moses1 {
// this class inherit functions from the "rapidity" class

cutsEmulator::cutsEmulator(){}
cutsEmulator::~cutsEmulator(){}


void cutsEmulator::setMinCosTheta(double s, double y, runParameters &runPrm) {
	double cosThetaStar_0 = 1.;
	double cosThetaStar_1 = 1.;

	// in very low masses, y can be larger than 2.5
	if (abs(y) <= runPrm.etaCut) {
		cosThetaStar_0 = abs(tanh(runPrm.etaCut-abs(y)));
	}
	else { cosThetaStar_0 = 0.; }
	if ((2.*runPrm.pTCut)/sqrt(s) <= 1.) {
		cosThetaStar_1 = sqrt(1-(2.*runPrm.pTCut/sqrt(s))*(2.*runPrm.pTCut/sqrt(s)));
	}
	else { cosThetaStar_1 = 0; }

	// find the minimum
	if (cosThetaStar_0 <= cosThetaStar_1) {m_cosThetaStar_min = cosThetaStar_0;}
	else {m_cosThetaStar_min = cosThetaStar_1;}
	
	// make sure it is <=1
	if (m_cosThetaStar_min > 1.) { m_cosThetaStar_min = 1.; }
}



double cutsEmulator::getModifiedCosThetaIntegral(double s, double y, runParameters &runPrm) {

	setMinCosTheta(s, y, runPrm);

	return ( 2*( (m_cosThetaStar_min) + (1./3.)*(m_cosThetaStar_min*m_cosThetaStar_min*m_cosThetaStar_min)) );
}



// overload to get forward-backward
double cutsEmulator::getModifiedCosThetaIntegral(double s, double y, runParameters &runPrm, int forwardBackward) {

        setMinCosTheta(s, y, runPrm);

	return ( (m_cosThetaStar_min * m_cosThetaStar_min) );
}


double cutsEmulator::getModifiedCosThetaIntegral(double s, double y, runParameters &runPrm, int forwardBackward, int SorA) {

        setMinCosTheta(s, y, runPrm);
	double A = m_cosThetaStar_min * m_cosThetaStar_min;
	double S = m_cosThetaStar_min + (1./3.)*(m_cosThetaStar_min*m_cosThetaStar_min*m_cosThetaStar_min);

	if (SorA == -1) { return ( A ); }
	else if (SorA == +1) { return ( S ); }
	else {
		cerr << "in cutsEmulator.cxx::getModifiedCosThetaIntegral() - invalid SorA=" << SorA << endl;
		return ( 0. );
	}
}



double cutsEmulator::getMinCosTheta(double s, double y, runParameters &runPrm) {
	setMinCosTheta(s, y, runPrm);
	return (m_cosThetaStar_min);
}


}

