/**************************************************/
/* a class for general analysis of pythia's event */
/**************************************************/
#include "kinematics.h"

using namespace std;
using namespace Pythia8;

namespace Moses1 {

kinematics::kinematics(){}
kinematics::~kinematics(){}


double kinematics::invMass(Particle &lepton, Particle &alepton) {
        return  (
                    sqrt( lepton.m()*lepton.m() + alepton.m()*alepton.m()
                          + 2 * lepton.e()*alepton.e()
                          - 2 * (lepton.px()*alepton.px() + lepton.py()*alepton.py() + lepton.pz()*alepton.pz())
                        )
                );
}

double kinematics::pT(Particle &lepton, Particle &alepton) {
        return  (
                   sqrt( (lepton.px() + alepton.px())*(lepton.px() + alepton.px())
                         + (lepton.py() + alepton.py())*(lepton.py() + alepton.py())
                       )
                );
}

double kinematics::y(Particle &lepton, Particle &alepton) {
        return  (
                   0.5*log(   (lepton.e() + alepton.e() + lepton.pz() + alepton.pz())
                            / (lepton.e() + alepton.e() - lepton.pz() - alepton.pz())
                          )
                );
}



bool kinematics::invMcut(Particle &lepton, Particle &alepton) {
	if (invMass(lepton,alepton) >= m_invMcut) { return true; }
	else { return false; }
}

bool kinematics::pTcut(Particle &lepton, Particle &alepton) {
	double pTl = sqrt( lepton.px()*lepton.px() + lepton.py()*lepton.py() );
	double pTal = sqrt( alepton.px()*alepton.px() + alepton.py()*alepton.py() );
	if (pTl >= m_pTcut   &&   pTal >= m_pTcut) { return true; }
	else { return false; }
}

bool kinematics::etaCut(Particle &lepton, Particle &alepton) {
	double pl  = sqrt( lepton.px()*lepton.px()   + lepton.py()*lepton.py()   + lepton.pz()*lepton.pz() );
	double pal = sqrt( alepton.px()*alepton.px() + alepton.py()*alepton.py() + alepton.pz()*alepton.pz() );
	double etal  = 0.5*log( (pl+lepton.pz())/(pl-lepton.pz()) );
	double etaal = 0.5*log( (pal+alepton.pz())/(pal-alepton.pz()) );
	if ( abs(etal)<m_etacut && abs(etaal)<m_etacut ) { return true; }
	else { return false; }
}

bool kinematics::pTsystemCut(Particle &lepton, Particle &alepton) {
	if ( pT(lepton,alepton) <= m_pTsystemCut ) {
		return true;
	}
	else { return false; }
}

bool kinematics::ySystemCut(Particle &lepton, Particle &alepton) {
	if ( y(lepton,alepton) > m_ySystemCut ) { return true; }
	else { return false; }
}


void kinematics::setCuts(double invMassCut, double pTransverseCut, double EtaCut, double pTsystemCut, double ySystemCut) {
        m_invMcut = invMassCut;
        m_pTcut = pTransverseCut;
        m_etacut = EtaCut;
	//------ leptons system cuts
	m_pTsystemCut = pTsystemCut;
	m_ySystemCut = ySystemCut;
}

bool kinematics::applyAllCuts(Particle &lepton, Particle &alepton) {
	if (lepton.id()+alepton.id()==0) {
		if ( invMcut(lepton,alepton) && pTcut(lepton,alepton) && etaCut(lepton,alepton) && pTsystemCut(lepton,alepton) && ySystemCut(lepton,alepton)) {
			return true;
		}
		else { return false; }
	}
	else { return false; }
}

double kinematics::getInvMcut() { return m_invMcut; }
double kinematics::getpTcut() { return m_pTcut; }
double kinematics::getEtacut() { return m_etacut; }
double kinematics::getpTsystemCut() { return m_pTsystemCut; }
double kinematics::getySystemCut() { return m_ySystemCut; }


}

