/**************************************** *******/
/* class to set new particle properties.        */
/* these are being added to the HEPPDT particle */
/* properties in a new struct with the first    */
/* member as pdtParticle struct                 */
/************************************************/
#include "mosesParticle.h"

using namespace std;

namespace Moses1 {


CUSTOMparticle::CUSTOMparticle(){}
CUSTOMparticle::~CUSTOMparticle(){}


// fill a SINGLE customParticle struct from a PDT particle structure (pythia) and then add new features manually
void CUSTOMparticle::setCUSTOMparticle(pdtParticle &pdt_particle, customParticle &custom_particle){
	
	// first, fill the PDT particle properties into the new struct
	// then, add some more properties to the new struct.
	// can add as many new properties as you want.
	// in this example, we add the weakIsospin and the corresponding weakI3 properties (quantum numbers),
	// which are not included in the PDT particle properties.

	custom_particle.pdt_prt = pdt_particle;

	if (pdt_particle.id == PDTDWN || pdt_particle.id == PDTSTR || pdt_particle.id == PDTBOT) { //d,s,b
		custom_particle.weakI3 = -0.5; custom_particle.Nc = 3;
	}
	if (pdt_particle.id == PDTUP || pdt_particle.id == PDTCHM || pdt_particle.id == PDTTOP) { //u,c,t
		custom_particle.weakI3 = 0.5; custom_particle.Nc = 3;
	}
	if (pdt_particle.id == PDTE || pdt_particle.id == PDTMU || pdt_particle.id == PDTTAU) { //e,mu,tau
		custom_particle.weakI3 = -0.5; custom_particle.Nc = 1;
	}
	if (pdt_particle.id == PDTNUE || pdt_particle.id == PDTNUMU || pdt_particle.id == PDTNUTAU) { //mu_e, mu_nu,mu_tau
		custom_particle.weakI3 = 0.5; custom_particle.Nc = 1;
	}
	else {cerr << "in mosesParticle.cxx: the supplied PDT particle id is not YET supported here" << endl;}
	
	// can add here more conditions to selectively fill other properties
}


// fill ALL customParticle struct from pythia and then add features manually
void CUSTOMparticle::setCUSTOMparticle(vector<pdtParticle> &pdt_particle, vector<customParticle> &custom_particle){
	
	// define a temp customParticle to temporarily store the properties,
	// and in each step push it back as a new member to the custom_particle vector
	customParticle tmp;

	// run over all PDT-particle-vector elements in a loop and fill the new vector SELECTIVELY !!!
	for (int i=0 ; i<(int)pdt_particle.size() ; i++) {

		switch(pdt_particle[i].id) {
			// first, fill in the PDT particle properties
			// then, add some more properties to the new structs.
			// can add as many new properties as you want,
			// in this example, we add the weakIsospin and the corresponding weakI3 properties (quantum numbers),
			// which are not included in the PDT particle properties.
			// finally, push_back the temporary struct as a new and modified element to the vector.

			// Pythia key to fundamental particles:
			// particle:    d u s c b t b' t' e- nu_e mu nu_mu tau- nu_tau tau' nu_tau'
			// id:          1 2 3 4 5 6 7  8  11 12   13 14    15   16     17   18
			case PDTDWN: //d
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = -0.5; tmp.Nc = 3;
				custom_particle.push_back(tmp); break;
			case PDTUP: //u
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = 0.5; tmp.Nc = 3;
				custom_particle.push_back(tmp); break;
			case PDTSTR: //s
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = -0.5; tmp.Nc = 3;
				custom_particle.push_back(tmp); break;
			case PDTCHM: //c
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = 0.5; tmp.Nc = 3;
				custom_particle.push_back(tmp); break;
			case PDTBOT: //b
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = -0.5; tmp.Nc = 3;
				custom_particle.push_back(tmp); break;
			case PDTTOP: //t
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = 0.5; tmp.Nc = 3;
				custom_particle.push_back(tmp); break;
			case PDTE: //e
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = -0.5; tmp.Nc = 1;
				custom_particle.push_back(tmp); break;
			case PDTNUE: //nu_e
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = 0.5; tmp.Nc = 1;
				custom_particle.push_back(tmp); break;
			case PDTMU: //mu
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = -0.5; tmp.Nc = 1;
				custom_particle.push_back(tmp); break;
			case PDTNUMU: //nu_mu
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = 0.5; tmp.Nc = 1;
				custom_particle.push_back(tmp); break;
			case PDTTAU: //tau
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = -0.5; tmp.Nc = 1;
				custom_particle.push_back(tmp); break;
			case PDTNUTAU: //nu_tau
				tmp.pdt_prt = pdt_particle[i]; tmp.weakI3 = 0.5; tmp.Nc = 1;
				custom_particle.push_back(tmp); break;
			default: break; // for other id's dont do anything !!! i.e the custom vector will be smaller and ad-hoc !!!
		}

		// again, can add here more conditions to selectively fill other properties
	}
}


}
