/************************************************************/
/* class to evaluate the user defined width                 */
/* the function here is taken from the "Z LINE SHAPE" paper */
/************************************************************/
#include "partialWidth.h"
using namespace std;

namespace Moses1 {

partialWidth::partialWidth(){}
partialWidth::~partialWidth(){}

// to return the partialWidth. This is for static width calculations, where it doesnt depend on sHat
double partialWidth::getpartialWidth(dcomplex gplus, dcomplex gminus,
					pdtParticle &propagator, customParticle &fermion,
					physicalParameters &physPrm) {
	double width = 0;

	if (propagator.id == PDTZ) { // is Z0 boson ?
		/*
		if (2*fermion.pdt_prt.mass >= propagator.mass) {
			cerr << "in partialWidth.cxx: products mass is larger than the propagator mass" << endl;
			exit(-1);
		}
		else {width = (physPrm.alpha/6)*fermion.Nc*propagator.mass*( abs(gplus)*abs(gplus) + abs(gminus)*abs(gminus) );}
		*/
		width = (physPrm.alpha/6)*fermion.Nc*propagator.mass*( abs(gplus)*abs(gplus) + abs(gminus)*abs(gminus) );
	}
	else { // if not Z0 boson
		cerr << "in partialWidth.cxx: the supplied propagator is not Z0" << endl;
		exit(-1);
	}
	return(width);
}



// for Z0/Z*/Z' and for gamma/gamma*. When the width depend on sHat and on the mode (mode=0: SM,Z', mode>0: ED)
double partialWidth::getpartialWidth(unsigned int nMode, double s,
				     pdtParticle &propagator, customParticle &outFermion,
				     physicalParameters &physPrm) {
	
	double mass = 0;
	double width = 0;

	if (propagator.id != PDTGAMMA && propagator.id != PDTZ && propagator.id != PDTZPRIME0) { // if not Z0/Z*/gamma/gamma* boson
		cerr << "in partialWidth.cxx: the supplied propagator is not Z0/gamma/Z*/gamma/gamma*/Z'0" << endl;
		exit(-1);
	}
	else {
		//if (s > 4*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass) {
                        if ((propagator.id==PDTZ || propagator.id==PDTZPRIME0) && nMode==0) { // SM Z0
				mass = propagator.mass;
				width = physPrm.Gmu*outFermion.Nc * mass*mass*mass / (3*physPrm.pi*sqrt(2)) *
					(
						outFermion.weakI3*outFermion.weakI3 -
						2*(outFermion.weakI3*outFermion.pdt_prt.charge*physPrm.sinThetaW2) +
						2*(outFermion.pdt_prt.charge*physPrm.sinThetaW2)*(outFermion.pdt_prt.charge*physPrm.sinThetaW2)
					);
				if(outFermion.pdt_prt.id == PDTTOP) { // top is not massless !!!
					if (s > 4*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass) {
						mass = sqrt(s);
						width = (1./(24.*physPrm.pi*sqrt(2.))) * physPrm.Gmu*outFermion.Nc * (propagator.mass*propagator.mass*mass) *
							sqrt(1. - 4.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mass*mass)) *
							(
								1. - 4.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mass*mass) +
								(2.*outFermion.weakI3 - 4.*outFermion.pdt_prt.charge * physPrm.sinThetaW2)*
								(2.*outFermion.weakI3 - 4.*outFermion.pdt_prt.charge * physPrm.sinThetaW2)*
								( 1. + 2.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mass*mass) )
							);
					//	cout << "in Z^0, top width = " << width << endl;
					}
				}
			}
			else if (propagator.id==PDTGAMMA && nMode==0) {width=0;} // SM photon
			else { // KK gamma* or Z*
				mass = sqrt(propagator.mass*propagator.mass + nMode*physPrm.mStar*nMode*physPrm.mStar);				
				
				if (propagator.id==PDTGAMMA) {
					width = outFermion.Nc*(physPrm.alpha/6) * mass * 2*(outFermion.pdt_prt.charge * outFermion.pdt_prt.charge);
					if(outFermion.pdt_prt.id == PDTTOP) { // top is not massless !!!
						width = outFermion.Nc*(physPrm.alpha/6.) * mass *
							sqrt( 1. - 4.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mass*mass) ) *
							(
								2.*(outFermion.pdt_prt.charge * outFermion.pdt_prt.charge) +
								4.*(outFermion.pdt_prt.charge * outFermion.pdt_prt.charge) *
								outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mass*mass)
							);
				//		cout << "in gamma^*, top width = " << width << endl;
					}
				}
				else if (propagator.id==PDTZ) {
					width = (1./(3*physPrm.pi*sqrt(2))) * physPrm.Gmu*outFermion.Nc * (propagator.mass*propagator.mass * mass) *
					(
						outFermion.weakI3*outFermion.weakI3 -
						2*(outFermion.weakI3 * outFermion.pdt_prt.charge * physPrm.sinThetaW2) +
						2*(outFermion.pdt_prt.charge * physPrm.sinThetaW2) * (outFermion.pdt_prt.charge * physPrm.sinThetaW2)
					);
					if(outFermion.pdt_prt.id == PDTTOP) { // top is not massless !!!
						width = (1./(24.*physPrm.pi*sqrt(2.))) * physPrm.Gmu*outFermion.Nc * (propagator.mass*propagator.mass * mass) *
							sqrt(1. - 4.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mass*mass)) *
							(
								1. - 4.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mass*mass) + 
								(2.*outFermion.weakI3 - 4.*outFermion.pdt_prt.charge * physPrm.sinThetaW2)*
								(2.*outFermion.weakI3 - 4.*outFermion.pdt_prt.charge * physPrm.sinThetaW2)*
								( 1. + 2.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mass*mass) )
							);
				//		cout << "in Z^*, top width = " << width << endl;
					}
				}
				width *= (sqrt(2)*sqrt(2)); // from the KK couplings !!!
			}
//		}
//		else {width=0;}
	}
	return(width);
}



double partialWidth::getpartialWidth(double s, pdtParticle &propagator, customParticle &outFermion,
				     physicalParameters &physPrm, double mZprime) {

        double mass = 0;
        double width = 0;

        if (propagator.id != PDTZ) { // if not Z0 boson for all properties EXCEPT for its mass
                cerr << "in partialWidth.cxx: the supplied propagator is not Z'0" << endl;
                exit(-1);
        }
        else {
//                if (s > 4*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass) {
			mass = propagator.mass;
			width = physPrm.Gmu*outFermion.Nc * mass*mass*mZprime / (3*physPrm.pi*sqrt(2)) *
				(
					outFermion.weakI3*outFermion.weakI3 -
					2*(outFermion.weakI3*outFermion.pdt_prt.charge*physPrm.sinThetaW2) +
					2*(outFermion.pdt_prt.charge*physPrm.sinThetaW2)*(outFermion.pdt_prt.charge*physPrm.sinThetaW2)
				);
		
			if(outFermion.pdt_prt.id == PDTTOP) { // top is not massless !!!
				width = (1./(24*physPrm.pi*sqrt(2))) * physPrm.Gmu*outFermion.Nc * (mass*mass * mZprime) *
					sqrt(1. - 4.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mZprime*mZprime)) *
					(
						1. - 4.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mZprime*mZprime) +
						(2.*outFermion.weakI3 - 4.*outFermion.pdt_prt.charge * physPrm.sinThetaW2)*
						(2.*outFermion.weakI3 - 4.*outFermion.pdt_prt.charge * physPrm.sinThetaW2)*
						( 1. + 2.*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass/(mZprime*mZprime) )
					);
			//	cout << "in Z^*, top width = " << width << endl;
			}
			
//		}
//		else {width=0;}
	}
	return(width);
}




//////////////////////////////////////////////// Gloun star////////////////////////////////////////////////
double   partialWidth::getpartialMassiveGWidth(unsigned int Lambda_I,unsigned int nMode, double s,
                                     pdtParticle &propagator, customParticle &outFermion,double &outHelicity1,double &outHelicity2,
                                     physicalParameters &physPrm, couplings &Gcouplings ) {

        double mass = 0;
        double width = 0;
        dcomplex couplings (0,0);

        if (propagator.id != PDTGLU) { // if not Gloun for all properties EXCEPT for its mass
                cerr << "in partialWidth.cxx: the supplied propagator is not Z'0" << endl;
                exit(-1);
        }
        else {
                if ((s > 4*outFermion.pdt_prt.mass*outFermion.pdt_prt.mass)||(s == -1.)) { //if s = 0 ignors this condition.
                                        mass =  physPrm.mGluonStar;
                                        //getMassiveGlounSchannelCouplings(unsigned int Lambda_I, double helicity1,double helicity2,pdtParticle &propagator,  customParticle &fermion1,physicalParameters &physPrm){
                                        couplings = Gcouplings.getMassiveGlounSchannelCouplings(Lambda_I, nMode, outHelicity1, outHelicity2, propagator, outFermion, outFermion, physPrm);
                                        width = real(couplings*conj(couplings)); // abs(couplings)
                                        //width *= ( physPrm.gQCD * mass/ 6. );
                                        //width *= ( physPrm.g_mGluonStar * mass/ 6. );
                                          width *= ( 1. * mass/ 6. );


                                                //      (


                                                //      );
                }
                else {width=0;}
        }
        return(width);
}



}

