 #include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TObject.h"
#include "TString.h"

#include <map>


TFile* f = NULL;

TTree* runSummaryTree = NULL;
TTree* controlTree    = NULL;


//---------------------------------- point it to where you want -------------------------------------
TString sRootDictDir  = "/srv01/tau/hod/moses-1.2.0/rootdict/";                      // mandatory
//TString sRootDictDir = gSystem->ExpandPathName("$MOSESSYS/rootdict/");             // another option
TString sDataFile = "/srv01/tau/hod/moses-1.2.0/data/mosesRunRecord.root";           // mandatory
//TString sDataFile = gSystem->ExpandPathName("$MOSESSYS/data/mosesRunRecord.root"); // another option
#include "/srv01/tau/hod/moses-1.2.0/rootdict/mapTObj.C"                             // mandatory
//----------------------------------------------------------------------------------------------------


void loadLibrary()
{
	gSystem->Load(sRootDictDir+"librootdict.so");
}


void readTree()
{
	TFile *f = new TFile(sDataFile);
	
	runSummaryTree = (TTree *)f->Get("runControl/runHeaderTree");
	
	// there is only one important master tree...
	controlTree    = (TTree *)f->Get("runControl/controlTree");
	
	// now make friends of the master tree
	/*
	controlTree->AddFriend("muon/muonTree", f);
	controlTree->AddFriend("electron/electronTree", f);
	controlTree->AddFriend("photon/photonTree", f);
	controlTree->AddFriend("neutrino/neutrinoTree", f);
	controlTree->AddFriend("meson/mesonTree", f);
	controlTree->AddFriend("baryon/baryonTree", f);
	controlTree->AddFriend("ClusterJet/ClusterJetTree", f);
	controlTree->AddFriend("CellJet/CellJetTree", f);
	*/
}


void printHeader(int errors)
{
	vector<int>*    beams_id;
    vector<double>* beams_m0;
	vector<double>* beams_p0;
	vector<double>* beams_pz;
	vector<string>* sProcessName;
	vector<int>*    nProcessCode;
	double eCM;
	int nEvents;
	//string sProcessName;
	//int nProcessCode;
	int nTried, nSelected, nAccepted;
	double sigma_mb, sigmaErr_mb;
	double mHatMin, mHatMax;
	runSummaryTree->SetBranchAddress("beamParticle_id", &beams_id);
	runSummaryTree->SetBranchAddress("beamParticle_m0", &beams_m0);
	runSummaryTree->SetBranchAddress("beamParticle_p0", &beams_p0);
	runSummaryTree->SetBranchAddress("beamParticle_pz", &beams_pz);
	runSummaryTree->SetBranchAddress("eCM",             &eCM);
	runSummaryTree->SetBranchAddress("nEvents",         &nEvents);
	runSummaryTree->SetBranchAddress("sProcessName",    &sProcessName);
	runSummaryTree->SetBranchAddress("nProcessCode",    &nProcessCode);
	runSummaryTree->SetBranchAddress("nTried",          &nTried);
	runSummaryTree->SetBranchAddress("nSelected",       &nSelected);
	runSummaryTree->SetBranchAddress("nAccepted",       &nAccepted);
	runSummaryTree->SetBranchAddress("sigma_mb",        &sigma_mb);
	runSummaryTree->SetBranchAddress("sigmaErr_mb",     &sigmaErr_mb);
	runSummaryTree->SetBranchAddress("mHatMin",         &mHatMin);
	runSummaryTree->SetBranchAddress("mHatMax",         &mHatMax);

	runSummaryTree->GetEntry(0);

	cout << "\n\n--------------------- begin run header ---------------------" << endl;
	if(errors>0) cout << "all errors in file: " << errors << endl;
	cout << "beam A id: " << (*beams_id)[0] << endl;
	cout << "\t m=" << (*beams_m0)[0] << endl;
	cout << "\t E=" << (*beams_p0)[0] << endl;
	cout << "\t pz=" << (*beams_pz)[0] << endl;
	cout << "beam B id: " << (*beams_id)[1] << endl;
	cout << "\t m=" << (*beams_m0)[1] << endl;
	cout << "\t E=" << (*beams_p0)[1] << endl;
	cout << "\t pz=" << (*beams_pz)[1] << endl;
	cout << "process (codes):" << endl;
	for(int p=0 ; p<nProcessCode->size() ; p++)
	{
		cout << "\tprocess[" << p << "]: " << (*sProcessName)[0] << " (" << (*nProcessCode)[0] << ")" << endl;
	}
	cout << "events:" << endl;
	cout << "\tnEvents(user): " << nEvents << endl;
	cout << "\tnTried: " << nTried << endl;
	cout << "\tnSelected: " << nSelected << endl;
	cout << "\tnAccepted: " << nAccepted << endl;
	cout << "cross section:" << endl;
	cout << "\tsigma[mb]: " << sigma_mb << " +- " << sigmaErr_mb << endl;
	cout << "\tE CM: " << eCM << endl;
	cout << "\tmHat min: " << mHatMin << endl;
	cout << "\tmHat max: " << mHatMax << endl;
	cout << "---------------------- end run header ----------------------\n\n" << endl;
}



void drawExamples()
{
	readTree();
	
	// draw the pT of the muons without a cut
	TCanvas* c1 = new TCanvas("c1","c1",800,600);
	c1->cd();
	c1->Draw();
	controlTree->Draw("sqrt(muonTree.px*muonTree.px+muonTree.py*muonTree.py)");
	
	// draw the pT of the muons with a cut: nJets>2
	TCanvas* c2 = new TCanvas("c2","c2",800,600);
	c2->cd();
	c2->Draw();
	TCut cut0  = "NCellJets>2";
	controlTree->Draw("sqrt(muonTree.px*muonTree.px + muonTree.py*muonTree.py)", cut0);
	
	TCanvas* c3 = new TCanvas("c3","c3",800,600);
	c3->SetLogy();
	c3->cd();
	c3->Draw();
	// draw the invarian mass of the muons with 4 cuts: nMuons=2, opposite charge, pTmu+>6 GeV, pTmu->6 GeV
	TCut cut1  = "Nmuons==2";
	TCut cut2  = "muonTree.id[0]+muonTree.id[1]==0";
	TCut cut3  = "sqrt(muonTree.px[0]*muonTree.px[0] + muonTree.py[0]*muonTree.py[0]) >= 0.";
	TCut cut4  = "sqrt(muonTree.px[1]*muonTree.px[1] + muonTree.py[1]*muonTree.py[1]) >= 0.";
	controlTree->Draw("sqrt(muonTree.m0[0]*muonTree.m0[0] + muonTree.m0[1]*muonTree.m0[1] + 2*muonTree.p0[0]*muonTree.p0[1] - 2*(muonTree.px[0]*muonTree.px[1] + muonTree.py[0]*muonTree.py[1] + muonTree.pz[0]*muonTree.pz[1]))", cut1+cut2+cut3+cut4);
	
	TCanvas* c4 = new TCanvas("c4","c4",800,600);
	c4->cd();
	c4->Draw();
	// draw the 2 jet finders results for nJets
	controlTree->Draw("NClusterJets:NCellJets", "", "BOX");
	
	TCanvas* c5 = new TCanvas("c5","c5",800,600);
	c5->cd();
	c5->Draw();
	// draw the 2 jet finders results for pT jets
	controlTree->Draw("sqrt(ClusterJetTree.px*ClusterJetTree.px + ClusterJetTree.py*ClusterJetTree.py):sqrt(CellJetTree.px*CellJetTree.px + CellJetTree.py*CellJetTree.py)", "", "BOX");
	
	TCanvas* c6 = new TCanvas("c6","c6",800,600);
	c6->cd();
	c6->Draw();
	// draw the ttbar invariant mass
	TCut cut5  = "hardProcessTree.id[4]+hardProcessTree.id[5]==0";
	controlTree->Draw("sqrt(hardProcessTree.m0[4]*hardProcessTree.m0[4] + hardProcessTree.m0[5]*hardProcessTree.m0[5] + 2*hardProcessTree.p0[4]*hardProcessTree.p0[5] - 2*(hardProcessTree.px[4]*hardProcessTree.px[5] + hardProcessTree.py[4]*hardProcessTree.py[5] + hardProcessTree.pz[4]*hardProcessTree.pz[5]))", cut5);
}


// An example of how to access the muonTree/ntuple variables
void eventLoop()
{
	readTree();

	// controls
	int nMuons, nElectrons, nPhotons, nNeutrinos, nMesons, nBaryons, nClusterJets, nCellJets;
	controlTree->SetBranchAddress("Nmuons",       &nMuons);
	controlTree->SetBranchAddress("Nelectrons",   &nElectrons);
	controlTree->SetBranchAddress("Nphotons",     &nPhotons);
	controlTree->SetBranchAddress("Nneutrinos",   &nNeutrinos);
	controlTree->SetBranchAddress("Nmesons",      &nMesons);
	controlTree->SetBranchAddress("Nbaryons",     &nBaryons);
	controlTree->SetBranchAddress("NClusterJets", &nClusterJets);
	controlTree->SetBranchAddress("NCellJets",    &nCellJets);

	//controlTree->Draw("sqrt(muonTree.px*muonTree.px + muonTree.py*muonTree.py) :NCellJets");
	
	// muons (called via controlTree)
	vector<double>* mu_p0;
	controlTree->SetBranchAddress("muon/muonTree.p0",&mu_p0);

	// electrons (called via controlTree)
	vector<double>* el_p0;
	controlTree->SetBranchAddress("electron/electronTree.p0",&el_p0);
	
	// photons (called via controlTree)
	vector<double>* pho_p0;
	controlTree->SetBranchAddress("photon/photonTree.p0",&pho_p0);
	
	// neutrinos (called via controlTree)
	vector<double>* nu_p0;
	controlTree->SetBranchAddress("neutrino/neutrinoTree.p0",&nu_p0);
	
	// mesons (called via controlTree)
	vector<double>* mes_p0;
	controlTree->SetBranchAddress("meson/mesonTree.p0",&mes_p0);

	// baryons (called via controlTree)
	vector<double>* bar_p0;
	controlTree->SetBranchAddress("baryon/baryonTree.p0",&bar_p0);
		
	// ClusterJets (called via controlTree)
	vector<double>* clusterjet_p0;
	controlTree->SetBranchAddress("ClusterJet/ClusterJetTree.p0",&clusterjet_p0);
		
	// CellJets (called via controlTree)
	vector<double>* celljet_p0;
	controlTree->SetBranchAddress("CellJet/CellJetTree.p0",&celljet_p0);


	int errors = 0;

	for (int i=0 ; i<controlTree->GetEntries() ; i++)
	{
		// get the controls:
		controlTree->GetEntry(i);
		
		// check the controls
		cout << "\tevent number=" << i+1 << ": \t\t(nMuons from control="<< nMuons << " /" << " ad hoc size="<< mu_p0->size() << ")" << endl;
		if (nMuons != mu_p0->size()) errors++;

		cout << "\tevent number=" << i+1 << ": \t\t(nElectrons from control="<< nElectrons << " /" << " ad hoc size="<< el_p0->size() << ")" << endl;
		if (nElectrons != el_p0->size()) errors++;	

		cout << "\tevent number=" << i+1 << ": \t\t(nPhotons from control="<< nPhotons << " /" << " ad hoc size="<< pho_p0->size() << ")" << endl;
		if (nPhotons != pho_p0->size()) errors++;

		cout << "\tevent number=" << i+1 << ": \t\t(nNeutrinos from control="<< nNeutrinos << " /" << " ad hoc size="<< nu_p0->size() << ")" << endl;
		if (nNeutrinos != nu_p0->size()) errors++;

		cout << "\tevent number=" << i+1 << ": \t\t(nMesons from control="<< nMesons << " /" << " ad hoc size="<< mes_p0->size() << ")" << endl;
		if (nMesons != mes_p0->size()) errors++;
		
		cout << "\tevent number=" << i+1 << ": \t\t(nBaryons from control="<< nBaryons << " /" << " ad hoc size="<< bar_p0->size() << ")" << endl;
		if (nBaryons != bar_p0->size()) errors++;
		
		cout << "\tevent number=" << i+1 << ": \t\t(nCellJets from control="<< nCellJets << " /" << " ad hoc size="<< celljet_p0->size() << ")" << endl;
		if (nCellJets != celljet_p0->size()) errors++;

		cout << "\tevent number=" << i+1 << ": \t\t(nClusterJets from control="<< nClusterJets << " /" << " ad hoc size="<< clusterjet_p0->size() << ")" << endl;
		if (nClusterJets != clusterjet_p0->size()) errors++;

		cout << "\n" << endl;
	}

	// read header
	printHeader(errors);
}

