// Example of how to read a root file and access the muonTree produced

// Make muonTree a global variable so that we can access it outside this function.
// You can then also access it from the command line. 
TFile* f = NULL;

TTree* runSummaryTree = NULL;
TTree* controlTree    = NULL;

TTree* muonTree     = NULL;
TTree* electronTree = NULL;
TTree* photonTree   = NULL;
TTree* neutrinoTree = NULL;
TTree* mesonTree    = NULL;
TTree* baryonTree   = NULL;

TTree* clusterJetTree = NULL;
TTree* cellJetTree    = NULL;


void readTree()
{
	TString filename = "/srv01/tau/hod/moses-1.2.0/data/mosesRunRecord.root";
	TFile *f = new TFile(filename);
	
	f->cd("runControl");
	runSummaryTree = (TTree *)f->Get("runControl/runHeaderTree");
	controlTree    = (TTree *)f->Get("runControl/controlTree");

	f->cd("muon");
	muonTree     = (TTree *)f->Get("muon/muonTree");
	f->cd("electron");
	electronTree = (TTree *)f->Get("electron/electronTree");
	f->cd("photon");
	photonTree   = (TTree *)f->Get("photon/photonTree");
	f->cd("neutrino");
	neutrinoTree = (TTree *)f->Get("neutrino/neutrinoTree");
	f->cd("meson");
	mesonTree    = (TTree *)f->Get("meson/mesonTree");
	f->cd("baryon");
	baryonTree   = (TTree *)f->Get("baryon/baryonTree");
	
	f->cd("ClusterJet");
	clusterJetTree = (TTree *)f->Get("ClusterJet/ClusterJetTree");
	f->cd("CellJet");
	cellJetTree    = (TTree *)f->Get("CellJet/CellJetTree");
	//muonTree->Draw("p0");
	//muonTree->Draw("mu_pt","evt_diMuonMass>10");
	//muonTree->Draw("mu_cosTheta","mu_pt>10");
}


void printHeader(int errors)
{
	vector<int>* beams_id;
    vector<double>* beams_m0;
	vector<double>* beams_p0;
	vector<double>* beams_pz;
	vector<string>* sProcessName;
	vector<int>*    nProcessCode;
	double eCM;
	int nEvents;
	//string sProcessName;
	//int nProcessCode;
	int nTried, nSelected, nAccepted;
	double sigma_mb, sigmaErr_mb;
	double mHatMin, mHatMax;
	runSummaryTree->SetBranchAddress("beamParticle_id", &beams_id);
	runSummaryTree->SetBranchAddress("beamParticle_m0", &beams_m0);
	runSummaryTree->SetBranchAddress("beamParticle_p0", &beams_p0);
	runSummaryTree->SetBranchAddress("beamParticle_pz", &beams_pz);
	runSummaryTree->SetBranchAddress("eCM",          &eCM);
	runSummaryTree->SetBranchAddress("nEvents",      &nEvents);
	runSummaryTree->SetBranchAddress("sProcessName", &sProcessName);
	runSummaryTree->SetBranchAddress("nProcessCode", &nProcessCode);
	runSummaryTree->SetBranchAddress("nTried",       &nTried);
	runSummaryTree->SetBranchAddress("nSelected",    &nSelected);
	runSummaryTree->SetBranchAddress("nAccepted",    &nAccepted);
	runSummaryTree->SetBranchAddress("sigma_mb",     &sigma_mb);
	runSummaryTree->SetBranchAddress("sigmaErr_mb", &sigmaErr_mb);
	runSummaryTree->SetBranchAddress("mHatMin", &mHatMin);
	runSummaryTree->SetBranchAddress("mHatMax", &mHatMax);

	runSummaryTree->GetEntry(0);

	cout << "\n\n--------------------- begin run header ---------------------" << endl;
	if(errors>0) cout << "all errors in file: " << errors << endl;
	cout << "beam A id: " << (*beams_id)[0] << endl;
	cout << "\t m=" << (*beams_m0)[0] << endl;
	cout << "\t E=" << (*beams_p0)[0] << endl;
	cout << "\t pz=" << (*beams_pz)[0] << endl;
	cout << "beam B id: " << (*beams_id)[1] << endl;
	cout << "\t m=" << (*beams_m0)[1] << endl;
	cout << "\t E=" << (*beams_p0)[1] << endl;
	cout << "\t pz=" << (*beams_pz)[1] << endl;
	cout << "process (codes):" << endl;
	for(int p=0 ; p<nProcessCode->size() ; p++)
	{
		cout << "\tprocess[" << p << "]: " << (*sProcessName)[0] << " (" << (*nProcessCode)[0] << ")" << endl;
	}
	cout << "events:" << endl;
	cout << "\tnEvents(user): " << nEvents << endl;
	cout << "\tnTried: " << nTried << endl;
	cout << "\tnSelected: " << nSelected << endl;
	cout << "\tnAccepted: " << nAccepted << endl;
	cout << "cross section:" << endl;
	cout << "\tsigma[mb]: " << sigma_mb << " +- " << sigmaErr_mb << endl;
	cout << "\tE CM: " << eCM << endl;
	cout << "\tmHat min: " << mHatMin << endl;
	cout << "\tmHat max: " << mHatMax << endl;
	cout << "---------------------- end run header ----------------------\n\n" << endl;
}



// An example of how to access the muonTree/ntuple variables
void eventLoop()
{
	readTree();

	// header
	vector<int>* beams_id;
	vector<double>* beams_m0;
	vector<double>* beams_p0;
	vector<double>* beams_pz;
	vector<string>* sProcessName;
	vector<int>*    nProcessCode;
	double eCM;
	int nEvents;
	//const char* sProcessName;
	//int nProcessCode;
	int nTried, nSelected, nAccepted;
	double sigma_mb, sigmaErr_mb;
	double mHatMin, mHatMax;
	
	runSummaryTree->SetBranchAddress("beamParticle_id", &beams_id);
	runSummaryTree->SetBranchAddress("beamParticle_m0", &beams_m0);
	runSummaryTree->SetBranchAddress("beamParticle_p0", &beams_p0);
	runSummaryTree->SetBranchAddress("beamParticle_pz", &beams_pz);
	runSummaryTree->SetBranchAddress("eCM",          &eCM);
	runSummaryTree->SetBranchAddress("nEvents",      &nEvents);
	runSummaryTree->SetBranchAddress("sProcessName", &sProcessName);
	runSummaryTree->SetBranchAddress("nProcessCode", &nProcessCode);
	runSummaryTree->SetBranchAddress("nTried",       &nTried);
	runSummaryTree->SetBranchAddress("nSelected",    &nSelected);
	runSummaryTree->SetBranchAddress("nAccepted",    &nAccepted);
	runSummaryTree->SetBranchAddress("sigma_mb",     &sigma_mb);
	runSummaryTree->SetBranchAddress("sigmaErr_mb", &sigmaErr_mb);
	runSummaryTree->SetBranchAddress("mHatMin", &mHatMin);
	runSummaryTree->SetBranchAddress("mHatMax", &mHatMax);

	// controls
	int nMuons, nElectrons, nPhotons, nNeutrinos, nMesons, nBaryons,
		nClusterJets, nCellJets;
	controlTree->SetBranchAddress("Nmuons",       &nMuons);
	controlTree->SetBranchAddress("Nelectrons",   &nElectrons);
	controlTree->SetBranchAddress("Nphotons",     &nPhotons);
	controlTree->SetBranchAddress("Nneutrinos",   &nNeutrinos);
	controlTree->SetBranchAddress("Nmesons",      &nMesons);
	controlTree->SetBranchAddress("Nbaryons",     &nBaryons);
	controlTree->SetBranchAddress("NClusterJets", &nClusterJets);
	controlTree->SetBranchAddress("NCellJets",    &nCellJets);


	// muons
	vector<double>* mu_p0;
	muonTree->SetBranchAddress("p0",&mu_p0);

	// photons
	vector<double>* pho_p0;
	photonTree->SetBranchAddress("p0",&pho_p0);
	
	// neutrinos
	vector<double>* nu_p0;
	neutrinoTree->SetBranchAddress("p0",&nu_p0);

	// electrons
	vector<double>* el_p0;
	electronTree->SetBranchAddress("p0",&el_p0);
	
	// mesons
	vector<double>* mes_p0;
	mesonTree->SetBranchAddress("p0",&mes_p0);

	// baryons
	vector<double>* bar_p0;
	baryonTree->SetBranchAddress("p0",&bar_p0);
		
	// ClusterJets
	vector<double>* clusterjet_p0;
	clusterJetTree->SetBranchAddress("p0",&clusterjet_p0);
		
	// CellJets
	vector<double>* celljet_p0;
	cellJetTree->SetBranchAddress("p0",&celljet_p0);


	int errors = 0;

	for (int i=0 ; i<controlTree->GetEntries() ; i++)
	{
		// get the controls:
		controlTree->GetEntry(i);
			
		// check the control:
		muonTree->GetEntry(i);
		cout << "\tevent number=" << i+1 << ": \t\t(nMuons from control="<< nMuons << " /" << " ad hoc size="<< mu_p0->size() << ")" << endl;
		if (nMuons != mu_p0->size()) errors++;

		electronTree->GetEntry(i);
		cout << "\tevent number=" << i+1 << ": \t\t(nElectrons from control="<< nElectrons << " /" << " ad hoc size="<< el_p0->size() << ")" << endl;
		if (nElectrons != el_p0->size()) errors++;	

		photonTree->GetEntry(i);
		cout << "\tevent number=" << i+1 << ": \t\t(nPhotons from control="<< nPhotons << " /" << " ad hoc size="<< pho_p0->size() << ")" << endl;
		if (nPhotons != pho_p0->size()) errors++;

		neutrinoTree->GetEntry(i);
		cout << "\tevent number=" << i+1 << ": \t\t(nNeutrinos from control="<< nNeutrinos << " /" << " ad hoc size="<< nu_p0->size() << ")" << endl;
		if (nNeutrinos != nu_p0->size()) errors++;

		mesonTree->GetEntry(i);
		cout << "\tevent number=" << i+1 << ": \t\t(nMesons from control="<< nMesons << " /" << " ad hoc size="<< mes_p0->size() << ")" << endl;
		if (nMesons != mes_p0->size()) errors++;
		
		baryonTree->GetEntry(i);
		cout << "\tevent number=" << i+1 << ": \t\t(nBaryons from control="<< nBaryons << " /" << " ad hoc size="<< bar_p0->size() << ")" << endl;
		if (nBaryons != bar_p0->size()) errors++;
		
		cellJetTree->GetEntry(i);
		cout << "\tevent number=" << i+1 << ": \t\t(nCellJets from control="<< nCellJets << " /" << " ad hoc size="<< celljet_p0->size() << ")" << endl;
		if (nCellJets != celljet_p0->size()) errors++;

		clusterJetTree->GetEntry(i);
		cout << "\tevent number=" << i+1 << ": \t\t(nClusterJets from control="<< nClusterJets << " /" << " ad hoc size="<< clusterjet_p0->size() << ")" << endl;
		if (nClusterJets != clusterjet_p0->size()) errors++;

		cout << "\n" << endl;
	}

	// read header
	printHeader(errors);
}

