/************************************************************/
/* an example program to demonstrate a basic use of pythia8 */
/* for an LHC setup                                         */
/************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace Moses1;
using namespace Pythia8;

int main() {
	// Generator. Process selection. LHC initialization. Histogram.
	Pythia pythia;

    //------------------------------------------------------
    // Chosen new PDF set; LHAPDF file name conventions.
    string pdfSet = "MRST2001lo.LHgrid";
    string pdfMember_str = "0";
    //------------------------------------------------------
	// PDF initialization
	pythia.readString("PDF:useLHAPDF = on");
	pythia.readString("PDF:LHAPDFset = " + pdfSet);
	pythia.readString("PDF:LHAPDFmember = " + pdfMember_str);
	// Allow extrapolation of PDF's beyond x and Q2 boundaries, at own risk.
	// Default behaviour is to freeze PDF's at boundaries.
	pythia.readString("PDF:extrapolateLHAPDF = on");
	// Need to change pT0Ref depending on choice of PDF.
	// One possibility: retune to same <n_charged>.
	//pythia.readString("MultipleInteractions:pT0Ref = 10");
	//------------------------------------------------------


	//----------------------------------------------------------------------------------------------
	// to switch on all the QCD process
	//pythia.readString("SoftQCD:all = on");

	// to switch on all the Top process
	// pythia.readString("Top:all = on");
	
	
	// to switch on the process ffbar--->gamma/Z
	pythia.readString("WeakSingleBoson:ffbar2gmZ = on");

	// rule out all gamma/Z decays except for muons
	pythia.readString("22:onMode = off"); // switch off all of the gamma decay modes
	pythia.readString("22:onIfAny = 13"); // switch on the gamma->mu-mu+ decay mode only
	pythia.readString("23:onMode = off"); // switch off all of the Z0 decay modes
	pythia.readString("23:onIfAny = 13"); // switch on the Z0->mu-mu+ decay mode only
	
	// phase space cuts
    pythia.readString("PhaseSpace:mHatMin = 20");
	pythia.readString("PhaseSpace:mHatMax = 200");
	

	// Initialization for LHC with time-based seed.
	pythia.readString("Random:setSeed = on");
	pythia.readString("Random:seed = 0");
    pythia.init(2212, 2212, 14000.);
	cout << "Initialization for LHC with time-based seed" << endl;

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();
	//----------------------------------------------------------------------------------------------

	// file
	commonOperations commonOps;
	string dirName = "/";
	
	string filePath_mosesEventrecord = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"mosesRunRecord.root");
	TFile *mosesEventRecordFile = new TFile(filePath_mosesEventrecord.c_str(),"recreate");	

	// chose what particle types to record in the record file
	//string record[] = {"muon", "electron", "photon", "neutrino", "meson", "baryon"};
	// build a vector of strings from this array
	//vector<string> sParticleRecords;
	//copy( record, record + sizeof(record)/sizeof(*record), inserter(sParticleRecords, sParticleRecords.end()) );


	// chose what particle types to record in the record file
	vector<string> sParticleRecords;
	sParticleRecords.push_back("muon");
	sParticleRecords.push_back("electron");
	sParticleRecords.push_back("photon");
	sParticleRecords.push_back("neutrino");
	sParticleRecords.push_back("meson");
	sParticleRecords.push_back("baryon");
	
	// create the mosesEvent instance (control everything in the run record)
	mosesEvent mosesevent(mosesEventRecordFile, sParticleRecords);
	
	// chose what jet types to record in the record file (if at all)
	// cluster jet (can be NULL if not needed):
	mosesClusterJetWrap* clusterJetWrap = new mosesClusterJetWrap("Lund", 2, 2, false, false);
	clusterJetWrap->setClusterJetAnalyzer(0., 50., 1, 0);
	// cell jet (can be NULL if not needed):
	mosesCellJetWrap* cellJetWrap = new mosesCellJetWrap(5., 50, 32, 2, 0, 0.5, 2., 0., 0);
	cellJetWrap->setCellJetAnalyzer(20., 0.7, 1.5);
	
	// tell mosesEvent to record also the jet(s)
	// can delete the next line or pass 2 NULL arguments instead, if jets are not needed
	mosesevent.passJetFinderWrappers(clusterJetWrap, cellJetWrap); // each one can be NULL if not needed
	
	
	// run
	int nEvents  = 10000;
	cout << "Events =? ";
	cin >> nEvents;
	int printMod = (int)nEvents/10;
	mosesevent.loopOverEvents(nEvents, printMod, pythia);
	
	pythia.statistics();
	mosesEventRecordFile->Close();
}

