#ifndef MOSESEVENT_h
#define MOSESEVENT_h

#include "mosesCommon.h"
#include "useHEPPDT.h"
#include "mosesJetFinderWrap.h"
#include "mosesParticleRecord.h"
#include "mosesJetRecord.h"
#include "mosesHardProcessRecord.h"

using namespace std;
using namespace Pythia8;


namespace Moses1 {

class mosesEvent
{
	public:
		mosesEvent(TFile* file, vector<string>& particleRecordNames);
		~mosesEvent();
		
		void passJetFinderWrappers(mosesClusterJetWrap* clusterJetWrap, mosesCellJetWrap* cellJetWrap);
		
		bool fillAllParticleRecords(Particle &particle, bool isHardProcess);
		bool fillAllJetRecords(Event& event);
		bool fillAllRecordTrees();
		bool clearAllParticleRecords();
		bool writeAllRecordTrees();
		bool writeControlTree();
		
		bool makeAllTreesFriends();

		bool isHardProcessStatus(int status);
		bool isHardProcess(Event& event, int index);

		bool loopOverEvents(int evts, int printMod, Pythia& pythia);

	private:
		void fillSupportedParticles();
		void fillSupportedJets();

		void pickRecords();
		void buildControlTree();

		bool fillControlTree();
		bool resetControls();

		void buildSummaryTree();
		bool fillSummaryTree(Pythia& pythia);
		bool writeSummaryTree();


		// general objects
		Pythia m_pythia;
		TFile* m_file;
		map<string, int> m_supportedParticles;
		map<string, int> m_supportedJets;
		vector<string>   m_particleRecordNames;
		vector<string>   m_jetRecordNames;
		bool m_bStatus;
		int m_evts;
		vector<int> m_vMothersList;
		TDirectory* m_dir;
		TTree* m_controlTree;

		// for the run summary
		TTree* m_runSummaryTree;
		vector<int>* m_beams_id;
        vector<double>* m_beams_m0;
        vector<double>* m_beams_p0;
        vector<double>* m_beams_pz;
		double m_eCM;
		int m_nEvents;
		vector<string>* m_sProcessName;
		vector<int>*    m_nProcessCode;
		int m_nTried, m_nSelected, m_nAccepted;
		double m_sigma_mb, m_sigmaErr_mb;
		double m_nMHatMin, m_nMHatMax;	
		
		// maps for the individual particle / jets operations
		map<string, mosesParticleRecord*> m_particlesMap;
		mosesClusterJetRecord* m_clusterJetRecord;
		mosesCellJetRecord*    m_cellJetRecord;
		mosesHardProcessRecord* m_hardProcessRecord;
		mosesClusterJetWrap* m_clusterJetWrap;
		mosesCellJetWrap*    m_cellJetWrap;
		map<string, bool> m_bRecord;
		map<string, int>  m_nRecords;
		int m_nHardPorcessParticles;
};


}

#endif
