#include "mosesJetFinderWrap.h"

using namespace std;

namespace Moses1
{




//------------ begin mosesClusterJetWrap
mosesClusterJetWrap::~mosesClusterJetWrap() { delete m_clusterJet; }
void mosesClusterJetWrap::setClusterJetPtr()
{
	m_clusterJet = new ClusterJet(m_measure, m_select, m_massSet, m_precluster, m_reassign);
	m_isAnalyzerSet = false;
}
void mosesClusterJetWrap::setClusterJetAnalyzer(double yScale, double pTscale, int nJetMin, int nJetMax)
{
        m_yScale  = yScale;
        m_pTscale = pTscale;
        m_nJetMin = nJetMin;
        m_nJetMax = nJetMax;
        m_isAnalyzerSet = true;
}
ClusterJet* mosesClusterJetWrap::getClusterJet() { return m_clusterJet; }
bool mosesClusterJetWrap::analyzeClusterJet( const Event& event )
{
        if(!m_isAnalyzerSet)
		{
			m_yScale = 10.;
			m_pTscale = 10.;
			m_nJetMin = 1;
			m_nJetMax = 0;
			setClusterJetAnalyzer(m_yScale, m_pTscale, m_nJetMin, m_nJetMax);
		}
        return m_clusterJet->analyze(event, m_yScale, m_pTscale, m_nJetMin, m_nJetMax, cout);
}
void mosesClusterJetWrap::getClusterJetSettings(string settingsName, int& settingsValue)
{
	if     (settingsName == "select")  settingsValue = m_select;
	else if(settingsName == "massSet") settingsValue = m_massSet;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
void mosesClusterJetWrap::getClusterJetSettings(string settingsName, bool& settingsValue)
{
	if     (settingsName == "precluster") settingsValue = m_precluster;
	else if(settingsName == "reassign")   settingsValue = m_reassign;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
void mosesClusterJetWrap::getClusterJetSettings(string settingsName, string& settingsValue)
{
	if(settingsName == "measure") settingsValue = m_measure;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
void mosesClusterJetWrap::getClusterJetAnalyzerSettings(string settingsName, double& settingsValue)
{
	if     (settingsName == "yScale")  settingsValue = m_yScale;
	else if(settingsName == "pTscale") settingsValue = m_pTscale;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
void mosesClusterJetWrap::getClusterJetAnalyzerSettings(string settingsName, int& settingsValue)
{
	if     (settingsName == "nJetMin") settingsValue = m_nJetMin;
	else if(settingsName == "nJetMax") settingsValue = m_nJetMax;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
//------------ end mosesClusterJetWrap




//------------ begin mosesCellJetWrap
mosesCellJetWrap::~mosesCellJetWrap() { delete m_cellJet; }
void mosesCellJetWrap::setCellJetPtr()
{		
    m_cellJet = new CellJet(m_etaMax, m_nEta, m_nPhi, m_select, m_smear, m_resolution, m_upperCut, m_threshold, m_rndmPtr);
    m_isAnalyzerSet = false;
}
void mosesCellJetWrap::setCellJetAnalyzer(double eTjetMin, double coneRadius, double eTseed)
{
        m_eTjetMin   = eTjetMin;
        m_coneRadius = coneRadius;
        m_eTseed     = eTseed;
        m_isAnalyzerSet = true;
}
CellJet* mosesCellJetWrap::getCellJet() { return m_cellJet; }
bool mosesCellJetWrap::analyzeCellJet( const Event& event )
{
        if(!m_isAnalyzerSet)
		{
			m_eTjetMin = 20.;
			m_coneRadius = 0.7;
			m_eTseed = 1.5;
			setCellJetAnalyzer(m_eTjetMin, m_coneRadius, m_eTseed);
		}
        return m_cellJet->analyze(event, m_eTjetMin, m_coneRadius, m_eTseed, cout);
}
void mosesCellJetWrap::getCellJetSettings(string settingsName, int& settingsValue)
{
	if     (settingsName == "nEta")   settingsValue = m_nEta;
	else if(settingsName == "nPhi")   settingsValue = m_nPhi;
	else if(settingsName == "select") settingsValue = m_select;
	else if(settingsName == "smear")  settingsValue = m_smear;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
void mosesCellJetWrap::getCellJetSettings(string settingsName, double& settingsValue)
{
	if     (settingsName == "etaMax")     settingsValue = m_etaMax;
	else if(settingsName == "resolution") settingsValue = m_resolution;
	else if(settingsName == "upperCut")   settingsValue = m_upperCut;
	else if(settingsName == "threshold")  settingsValue = m_threshold;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
void mosesCellJetWrap::getCellJetSettings(string settingsName, Rndm* settingsValue)
{
	if     (settingsName == "rndmPtr")  settingsValue = m_rndmPtr;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
void mosesCellJetWrap::getCellJetAnalyzerSettings(string settingsName, double& settingsValue)
{
	if     (settingsName == "eTjetMin")   settingsValue = m_eTjetMin;
	else if(settingsName == "coneRadius") settingsValue = m_coneRadius;
	else if(settingsName == "eTseed")     settingsValue = m_eTseed;
	else cerr << "WARNING: setting name '" << settingsName << "' is unknown" << endl;
}
//------------ end mosesCellJetWrap
}
