#include "mosesJetRecord.h"

using namespace std;

namespace Moses1 {


// cluster Jet
mosesClusterJetRecord::mosesClusterJetRecord(string name, TFile* file)
{
	initialize(name, file);
}

mosesClusterJetRecord::~mosesClusterJetRecord(){}

void mosesClusterJetRecord::initialize(string name, TFile* file)
{
	m_name = name;

    m_dir = file->mkdir(m_name.c_str());
	m_dir->cd();
    cout << "built the " << m_name << " dir, ";

	m_isHardProcess = new vector<int>;
	m_p0   = new vector<double>;
	m_px   = new vector<double>;
	m_py   = new vector<double>;
	m_pz   = new vector<double>;

    string dataTreeName  = name+"Tree";
    string dataTreeTitle = name;
    m_jetDataTree = new TTree(dataTreeName.c_str(),dataTreeTitle.c_str());
	m_jetDataTree->Branch("isHardProcess", &m_isHardProcess);
    m_jetDataTree->Branch("p0", &m_p0);
    m_jetDataTree->Branch("px", &m_px);
    m_jetDataTree->Branch("py", &m_py);
    m_jetDataTree->Branch("pz", &m_pz);
    cout << "data tree " << endl;;
}

bool mosesClusterJetRecord::fillRecordTree()
{
	m_jetDataTree->Fill();
	return true;
}

bool mosesClusterJetRecord::writeRecordTree()
{
	m_dir->cd();
	m_jetDataTree->Write();
	return true;
}

bool mosesClusterJetRecord::clearJetRecord()
{
	if(m_p0!=NULL && m_p0->size()!=0) m_p0->clear();
	if(m_px!=NULL && m_px->size()!=0) m_px->clear();
	if(m_py!=NULL && m_py->size()!=0) m_py->clear();
	if(m_pz!=NULL && m_pz->size()!=0) m_pz->clear();

	if(m_isHardProcess!=NULL && m_isHardProcess->size()!=0) m_isHardProcess->clear();
	
	return true;
}

bool mosesClusterJetRecord::isHardProcessStatus(int absStatus)
{
	// Particle::status() = 21 - 29 : particles of the hardest subprocess
	// Particle::status() = 31 - 39 : particles of subsequent subprocesses
	// Particle::status() = 41 - 49 : articles produced by initial-state-showers
	// Particle::status() = 51 - 59 : particles produced by final-state-showers
    if     ( absStatus<=29 && absStatus>=21 ) return true;
	else if( absStatus<=39 && absStatus>=31 ) return true; 
    else if( absStatus<=49 && absStatus>=41 ) return true;
	else if( absStatus<=59 && absStatus>=51 ) return true;
	return false;
}

void mosesClusterJetRecord::findHardProcessJets(Event& event, ClusterJet* clusterJet)
{
	// initialize to false
	m_hardProcessJets.clear();
	for(int i=0 ; i<(int)clusterJet->size() ; i++) m_hardProcessJets.push_back(false);

	int jetIndex = 0;
		
	for(int i=0 ; i<event.size() ; i++)
	{
		if( isHardProcessStatus( event[i].status() ) )
		{
			jetIndex = clusterJet->jetAssignment(i); // gives the index of the jet that the particle i of the event record belongs to
			//cout << "jetAssignment(" << event[i].name() << ") = " << jetIndex << "/" << clusterJet->size() << endl;
			if( jetIndex>-1 && jetIndex<(int)clusterJet->size() ) m_hardProcessJets[jetIndex] = true;
		}
	}
}

bool mosesClusterJetRecord::fillJetRecord( Event& event, ClusterJet* clusterJet )
{
	Vec4 pJet;
	int nIsHardProcess = 0;
	
	for(int j=0 ; j<clusterJet->size() ; j++)
	{
		pJet = clusterJet->p(j);
		nIsHardProcess = ( m_hardProcessJets[j] ) ? 1 : 0; // ?????????????????????????????????????????????????????
		
		m_p0->push_back( pJet.e() );
		m_px->push_back( pJet.px() );
		m_py->push_back( pJet.py() );
		m_pz->push_back( pJet.pz() );
		
		m_isHardProcess->push_back( nIsHardProcess );
	}
	
	return true;
}








// cell jet
mosesCellJetRecord::mosesCellJetRecord(string name, TFile* file)
{
	initialize(name, file);
}

mosesCellJetRecord::~mosesCellJetRecord(){}

void mosesCellJetRecord::initialize(string name, TFile* file)
{
	m_name = name;

    m_dir = file->mkdir(m_name.c_str());
	m_dir->cd();
    cout << "built the " << m_name << " dir, ";

	m_p0   = new vector<double>;
	m_px   = new vector<double>;
	m_py   = new vector<double>;
	m_pz   = new vector<double>;

	m_m0           = new vector<double>;
	m_p0Massless   = new vector<double>;
	m_pxMassless   = new vector<double>;
	m_pyMassless   = new vector<double>;
	m_pzMassless   = new vector<double>;
	m_eT           = new vector<double>;
	m_etaCenter    = new vector<double>;
	m_phiCenter    = new vector<double>;
	m_etaWeighted  = new vector<double>;
	m_phiWeighted  = new vector<double>;
	m_multiplicity = new vector<int>;

    string dataTreeName  = name+"Tree";
    string dataTreeTitle = name;
    m_jetDataTree = new TTree(dataTreeName.c_str(),dataTreeTitle.c_str());
    m_jetDataTree->Branch("p0", &m_p0);
    m_jetDataTree->Branch("px", &m_px);
    m_jetDataTree->Branch("py", &m_py);
    m_jetDataTree->Branch("pz", &m_pz);
	
	m_jetDataTree->Branch("m0", &m_m0);
    m_jetDataTree->Branch("p0Massless",   &m_p0Massless);
    m_jetDataTree->Branch("pxMassless",   &m_pxMassless);
    m_jetDataTree->Branch("pyMassless",   &m_pyMassless);
    m_jetDataTree->Branch("pzMassless",   &m_pzMassless);
	m_jetDataTree->Branch("eT",           &m_eT);
	m_jetDataTree->Branch("etaCenter",    &m_etaCenter);
	m_jetDataTree->Branch("phiCenter",    &m_phiCenter);
	m_jetDataTree->Branch("etaWeighted",  &m_etaWeighted);
	m_jetDataTree->Branch("phiWeighted",  &m_phiWeighted);
	m_jetDataTree->Branch("multiplicity", &m_multiplicity);
	
    cout << "data tree " << endl;;
}

bool mosesCellJetRecord::fillRecordTree()
{
	m_jetDataTree->Fill();
	return true;
}

bool mosesCellJetRecord::writeRecordTree()
{
	m_dir->cd();
	m_jetDataTree->Write();
	return true;
}

bool mosesCellJetRecord::clearJetRecord()
{
	if(m_p0!=NULL && m_p0->size()!=0) m_p0->clear();
	if(m_px!=NULL && m_px->size()!=0) m_px->clear();
	if(m_py!=NULL && m_py->size()!=0) m_py->clear();
	if(m_pz!=NULL && m_pz->size()!=0) m_pz->clear();
	
	if(m_p0Massless!=NULL && m_p0Massless->size()!=0) m_p0Massless->clear();
	if(m_pxMassless!=NULL && m_pxMassless->size()!=0) m_pxMassless->clear();
	if(m_pyMassless!=NULL && m_pyMassless->size()!=0) m_pyMassless->clear();
	if(m_pzMassless!=NULL && m_pzMassless->size()!=0) m_pzMassless->clear();
	if(m_etaCenter!=NULL && m_etaCenter->size()!=0) m_etaCenter->clear();
	if(m_phiCenter!=NULL && m_phiCenter->size()!=0) m_phiCenter->clear();
	if(m_etaWeighted!=NULL && m_etaWeighted->size()!=0) m_etaWeighted->clear();
	if(m_phiWeighted!=NULL && m_phiWeighted->size()!=0) m_phiWeighted->clear();
	if(m_multiplicity!=NULL && m_multiplicity->size()!=0) m_multiplicity->clear();
	if(m_eT!=NULL && m_eT->size()!=0) m_eT->clear();
	if(m_m0!=NULL && m_m0->size()!=0) m_m0->clear();
	
	return true;
}

bool mosesCellJetRecord::fillJetRecord( Event& event, CellJet* cellJet )
{
	
	Vec4 pJet, pJetMassless;
	double jetMass, eT, etaCenter, phiCenter, etaWeighted, phiWeighted;
	int multiplicity;
	
	for(int j=0 ; j<cellJet->size() ; j++)
	{
		pJet = cellJet->pMassive(j);
		jetMass = cellJet->m(j);
		pJetMassless = cellJet->pMassless(j);
		eT = cellJet->eT(j);
		etaCenter = cellJet->etaCenter(j);
		phiCenter = cellJet->phiCenter(j);
		etaWeighted = cellJet->etaWeighted(j);
		phiWeighted = cellJet->phiWeighted(j);
		multiplicity = cellJet->multiplicity(j);

		m_p0->push_back( pJet.e() );
		m_px->push_back( pJet.px() );
		m_py->push_back( pJet.py() );
		m_pz->push_back( pJet.pz() );
		
		m_m0->push_back( jetMass );
		m_p0Massless->push_back( pJetMassless.e() );
		m_pxMassless->push_back( pJetMassless.px() );
		m_pyMassless->push_back( pJetMassless.py() );
		m_pzMassless->push_back( pJetMassless.pz() );
		m_eT->push_back( eT );
		m_etaCenter->push_back( etaCenter );
		m_phiCenter->push_back( phiCenter );
		m_etaWeighted->push_back( etaWeighted );
		m_phiWeighted->push_back( phiWeighted );
		m_multiplicity->push_back( multiplicity );
	}
	
	return true;
}



}
